@@ -1,4 +1,4 @@
-#!/bin/bash - 
+#!/bin/bash -
 
 # Copyright(c) 2011 LinBox
 # Written by Brice Boyer (briceboyer) <boyer.brice@gmail.com>
@@ -15,12 +15,12 @@
 # TODO : rpath instead of LD_LIBRARY_PATH ?
 
 #############################
-## Only for stable fetching 
+## Only for stable fetching
 #############################
-#### stable .tar.gz 
-STABLE_LB=1.6.2
-STABLE_FFLAS=2.4.0
-STABLE_GIVARO=4.1.0
+#### stable .tar.gz
+STABLE_LB=1.6.3
+STABLE_FFLAS=2.4.1
+STABLE_GIVARO=4.1.1
 STABLE_OPENBLAS=0.3.6
 STABLE_BLIS=0.5.2
 MD5SUFF=md5
@@ -60,7 +60,7 @@ OPENBLAS=""
 OPENBLAS_VAR="false"
 BLIS=""
 BLIS_VAR="false"
-MAKEOPT= 
+MAKEOPT=
 MAKE_VAR=""
 
 DONE="\033[0;36m done !\033[0m"
@@ -71,13 +71,13 @@ BEG="\033[1;32m * \033[0m"
 #########
 
 die() {
-    echo -ne "\n\033[1;31m * \033[0mfailed" ;   
+    echo -ne "\n\033[1;31m * \033[0mfailed" ;
     if [[ -n $1 ]] ; then
 	echo " ($1)"
-    else 
+    else
 	echo "."
     fi
-    exit -1 ;  
+    exit -1 ;
 }
 
 cool() {
@@ -100,10 +100,10 @@ help() {
     echo " --prefix=MY/PATH      : install all libraries under MY/PATH."
     echo "                         Default : /tmp"
     echo
-    echo " >> Libraries to search for <<" 
-    echo 
+    echo " >> Libraries to search for <<"
+    echo
     echo " If some library cannot be linked, don't forget to export LD_LIBRARY_PATH ! "
-    echo 
+    echo
     echo " --with-gmp=GMP/PATH   : tell where gmp is."
     echo "                         Default : /usr, /usr/local. No argument is Default"
     echo " --with-blas-libs=BLAS/PATH : same as GMP for BLAS. (will check anyway)"
@@ -131,16 +131,16 @@ help() {
     echo "                         Default : enabled."
     echo " --enable-drivers      : build with drivers support."
     echo "                         Default : disabled."
-    echo 
+    echo
     echo " >> calling helllp <<"
-    echo 
+    echo
     echo " --help, -h, -?        : print help and exit."
 }
 
 
 
 ############
-#  parser  # 
+#  parser  #
 ############
 
 # Recover command line, with double-quotes
@@ -161,7 +161,7 @@ echo  "$0 $CMDLINE" | tee -a linbox-auto-install.log
 for i in "$@" ; do
     case "$i" in
 		# switches
-	"--help"|"-h"|"-?") 
+	"--help"|"-h"|"-?")
 	help
 	exit 0
 	;;
@@ -261,10 +261,10 @@ for i in "$@" ; do
 	case "$QUI" in
 	    "--stable")
 	    OK=2
-	    [[ "x$QUOI" = "xyes" ]]  &&  OK=1  
-	    [[ "x$QUOI" = "xno"  ]]  &&  OK=0   
-	    if [[ "$OK" = "2" ]] ; then 
-		echo "stable=[yes/no] !" ; help ; exit -1 ; 
+	    [[ "x$QUOI" = "xyes" ]]  &&  OK=1
+	    [[ "x$QUOI" = "xno"  ]]  &&  OK=0
+	    if [[ "$OK" = "2" ]] ; then
+		echo "stable=[yes/no] !" ; help ; exit -1 ;
 	    fi
 	    [[ "$OK" = "1" ]] && STABLE_VAR="true" || STABLE_VAR="false"
 	    ;;
@@ -318,7 +318,7 @@ for i in "$@" ; do
 	    [[ "$QUOI" =~ y|yes|Y|1 ]] && OK=1 || OK=0
 	    if		[ "x$OPENBLAS_VAR" = "xtrue"  -a "OK" = "0" ] ; then  echo "openblas or not openblas ?" ;      help ; exit -1; fi
 	    if		[ "x$OPENBLAS_VAR" = "xfalse" -a "OK" = "1" ] ; then  echo "openblas or not openblas ?" ;      help ; exit -1; fi
-	    if	[[ "x$OK" = "x1" ]] ; then  
+	    if	[[ "x$OK" = "x1" ]] ; then
 		OPENBLAS=$QUI ; OPENBLAS_VAR="true" ;
 	    else
 		OPENBLAS_VAR="false" ;
@@ -341,7 +341,7 @@ for i in "$@" ; do
 	    if [[ "x$OK" = "x1" ]] ; then
 		WARNINGS=$QUI ; WARNING_VAR="true"
 	    else
-		WARNING_VAR="false" 
+		WARNING_VAR="false"
 	    fi
 	    [[ "x$QUOI" = "xfull" ]] && WARNINGS=$i
 	    ;;
@@ -349,19 +349,19 @@ for i in "$@" ; do
 	    [[ "$QUOI" =~ y|yes|Y|1 ]] && OK=1 || OK=0
 	    if		[ "x$DEBUG_VAR" = "xtrue"  -a "OK" = "0"  ] ; then  echo "debug or not debug ?" ;      help ; exit -1; fi
 	    if		[ "x$DEBUG_VAR" = "xfalse" -a "OK" = "1"  ] ; then  echo "debug or not debug ?" ;      help ; exit -1; fi
-	    if		[[ "x$OK" = "x1" ]] ; then  
+	    if		[[ "x$OK" = "x1" ]] ; then
 		DEBUG=$QUI ; DEBUG_VAR="true"
 	    else
-		DEBUG_VAR="false" 
+		DEBUG_VAR="false"
 	    fi
 	    ;;
 	    "--enable-check")
 	    [[ "$QUOI" =~ y|yes|Y|1 ]] && OK=1 || OK=0
 	    if		[ "x$CHECK_VAR" = "xtrue"  -a "OK" = "0"  ] ; then  echo "check or not check ?" ;      help ; exit -1; fi
 	    if		[ "x$CHECK_VAR" = "xfalse" -a "OK" = "1"  ] ; then  echo "check or not check ?" ;      help ; exit -1; fi
-	    if		[[ "x$OK" = "x1" ]] ; then 
-		CHECK=$QUI ; CHECK_VAR="true" 
-	    else 
+	    if		[[ "x$OK" = "x1" ]] ; then
+		CHECK=$QUI ; CHECK_VAR="true"
+	    else
 		CHECK_VAR="false"
 	    fi
 	    ;;
@@ -369,10 +369,10 @@ for i in "$@" ; do
 	    [[ "$QUOI" =~ y|yes|Y|1 ]] && OK=1 || OK=0
 	    if		[ "x$OPTIM_VAR" = "xtrue"  -a "OK" = "0"  ] ; then  echo "drivers or not drivers ?" ;      help ; exit -1; fi
 	    if		[ "x$OPTIM_VAR" = "xfalse" -a "OK" = "1"  ] ; then  echo "drivers or not drivers ?" ;      help ; exit -1; fi
-	    if		[[ "x$OK" = "x1" ]] ; then 
+	    if		[[ "x$OK" = "x1" ]] ; then
 		DRIV=$QUI ; DRIV_VAR="true"
 	    else
-		DRIV_VAR="false" 
+		DRIV_VAR="false"
 	    fi
 	    ;;
 	    *)
@@ -414,22 +414,22 @@ if [ ! \( -x autogen.sh -o -x configure \) ] ; then
 	else
 	    wget https://github.com/linbox-team/linbox/releases/download/v${STABLE_LB}/linbox-${STABLE_LB}.tar.gz >/dev/null 2>&1 || die
 	    [ -f linbox-${STABLE_LB}.tar.gz ] &&  cool || die
-	    echo -ne "${BEG}fetching md5sum" | tee -a linbox-auto-install.log; 
+	    echo -ne "${BEG}fetching md5sum" | tee -a linbox-auto-install.log;
 	    wget --no-check-certificate https://github.com/linbox-team/linbox/releases/download/v${STABLE_LB}/linbox-${STABLE_LB}.tar.gz.${MD5SUFF} >/dev/null 2>&1 || die
 	    cool| tee -a linbox-auto-install.log
 	    echo -ne "${BEG}"
 	    md5sum -c linbox-${STABLE_LB}.tar.gz.${MD5SUFF} || die
 	fi
 	OK=0
-	echo -en "${BEG}extracting LinBox..."
+	echo -en "${BEG}extracting LinBox..."| tee -a linbox-auto-install.log
 	decompress linbox-${STABLE_LB}.tar.gz  && OK=1
-	[ "$OK" = "1" ] &&  cool | tee -a linbox-auto-install.log  || die 
-	cd linbox-${STABLE_LB} &&  cool   || die 
+	[ "$OK" = "1" ] &&  cool | tee -a linbox-auto-install.log  || die
+	cd linbox-${STABLE_LB} &&  cool   || die
     else
 	OK=0 ;
 	git clone --depth 1 https://github.com/linbox-team/linbox.git 2>&1 >/dev/null && OK=1
 	[ "$OK" = "1" ] &&  cool | tee -a linbox-auto-install.log || die
-	cd linbox &&  cool   || die 
+	cd linbox &&  cool   || die
     fi
     mv ../linbox-auto-install.log . || die
 fi
@@ -465,8 +465,8 @@ cd build ;
 echo -en "${BEG}fetching Givaro..."| tee -a ../linbox-auto-install.log
 if [ "$STABLE_VAR" = "true" ]; then
     if [ -f givaro-${STABLE_GIVARO}.tar.gz ] ; then
-	echo -ne " already there!\n"
-	echo -ne "${BEG}fetching md5sum" ; 
+	echo -ne " already there!\n"| tee -a ../linbox-auto-install.log
+	echo -ne "${BEG}fetching md5sum"| tee -a ../linbox-auto-install.log ;
 	[ -f givaro-${STABLE_GIVARO}.tar.gz.${MD5SUFF} ] && rm givaro-${STABLE_GIVARO}.tar.gz.${MD5SUFF} ;
 	wget --no-check-certificate https://github.com/linbox-team/givaro/releases/download/v${STABLE_GIVARO}/givaro-${STABLE_GIVARO}.tar.gz.${MD5SUFF} >/dev/null 2>&1 || die
 	[ -f givaro-${STABLE_GIVARO}.tar.gz.${MD5SUFF} ] || die
@@ -476,7 +476,7 @@ if [ "$STABLE_VAR" = "true" ]; then
     else
 	wget --no-check-certificate https://github.com/linbox-team/givaro/releases/download/v${STABLE_GIVARO}/givaro-${STABLE_GIVARO}.tar.gz >/dev/null 2>&1 || die
 	[ -f givaro-${STABLE_GIVARO}.tar.gz ] &&  cool || die
-	echo -ne "${BEG}fetching md5sum" ; 
+	echo -ne "${BEG}fetching md5sum"| tee -a ../linbox-auto-install.log ;
 	wget --no-check-certificate https://github.com/linbox-team/givaro/releases/download/v${STABLE_GIVARO}/givaro-${STABLE_GIVARO}.tar.gz.${MD5SUFF} >/dev/null 2>&1 || die
 	cool| tee -a ../linbox-auto-install.log
 	echo -ne "${BEG}"
@@ -485,7 +485,7 @@ if [ "$STABLE_VAR" = "true" ]; then
 else
     OK=0 ;
     git clone --depth 1 https://github.com/linbox-team/givaro.git 2>&1 >/dev/null && OK=1
-    [ "$OK" = "1" ] &&  cool | tee -a ../linbox-auto-install.log || die 
+    [ "$OK" = "1" ] &&  cool | tee -a ../linbox-auto-install.log || die
 fi
 
 ### Fflas-ffpack ###
@@ -493,8 +493,8 @@ fi
 echo -en "${BEG}fetching Fflas-Ffpack..."| tee -a ../linbox-auto-install.log
 if [ "$STABLE_VAR" = "true" ]; then
     if [ -f fflas-ffpack-${STABLE_FFLAS}.tar.gz ] ; then
-	echo -ne " already there!\n"
-	echo -ne "${BEG}fetching md5sum" ; 
+	echo -ne " already there!\n"| tee -a ../linbox-auto-install.log
+	echo -ne "${BEG}fetching md5sum"| tee -a ../linbox-auto-install.log ;
 	[ -f fflas-ffpack-${STABLE_FFLAS}.tar.gz.${MD5SUFF} ] && rm fflas-ffpack-${STABLE_FFLAS}.tar.gz.${MD5SUFF} ;
 	wget --no-check-certificate https://github.com/linbox-team/fflas-ffpack/releases/download/v${STABLE_FFLAS}/fflas-ffpack-${STABLE_FFLAS}.tar.gz.${MD5SUFF} >/dev/null 2>&1 || die
 	[ -f fflas-ffpack-${STABLE_FFLAS}.tar.gz.${MD5SUFF} ] &&  cool || die
@@ -504,7 +504,7 @@ if [ "$STABLE_VAR" = "true" ]; then
     else
 	wget --no-check-certificate https://github.com/linbox-team/fflas-ffpack/releases/download/v${STABLE_FFLAS}/fflas-ffpack-${STABLE_FFLAS}.tar.gz >/dev/null 2>&1 || die
 	[ -f fflas-ffpack-${STABLE_FFLAS}.tar.gz ] &&  cool || die
-	echo -ne "${BEG}fetching md5sum" ; 
+	echo -ne "${BEG}fetching md5sum"| tee -a ../linbox-auto-install.log ;
 	wget --no-check-certificate https://github.com/linbox-team/fflas-ffpack/releases/download/v${STABLE_FFLAS}/fflas-ffpack-${STABLE_FFLAS}.tar.gz.${MD5SUFF} >/dev/null 2>&1 || die
 	cool
 	echo -ne "${BEG}"
@@ -522,7 +522,7 @@ if [ "$OPENBLAS_VAR" = "true" ]; then
     echo -en "${BEG}fetching OpenBlas..."| tee -a ../linbox-auto-install.log
     if [ "$STABLE_VAR" = "true" ]; then
 	if [ -f v${STABLE_OPENBLAS}.tar.gz ] ; then
-	    echo -ne " already there!\n"
+	    echo -ne " already there!\n"| tee -a ../linbox-auto-install.log
 	else
 	    wget --no-check-certificate http://github.com/xianyi/OpenBLAS/archive/v${STABLE_OPENBLAS}.tar.gz >/dev/null 2>&1 || die
 	    [ -f v${STABLE_OPENBLAS}.tar.gz ] &&  cool || die
@@ -539,7 +539,7 @@ if [ "$BLIS_VAR" = "true" ]; then
     echo -en "${BEG}fetching FLAME/Blis..."| tee -a ../linbox-auto-install.log
     if [ "$STABLE_VAR" = "true" ]; then
 	if [ -f ${STABLE_BLIS}.tar.gz ] ; then
-	    echo -ne " already there!\n"
+	    echo -ne " already there!\n"| tee -a ../linbox-auto-install.log
 	else
 	    wget --no-check-certificate https://github.com/flame/blis/archive/${STABLE_BLIS}.tar.gz >/dev/null 2>&1 || die
 	    [ -f ${STABLE_BLIS}.tar.gz ] &&  cool || die
@@ -561,7 +561,7 @@ OK=0
 if [ "$STABLE_VAR" = "true" ]; then
     echo -en "${BEG}extracting Givaro..."| tee -a ../linbox-auto-install.log
     decompress givaro-${STABLE_GIVARO}.tar.gz  && OK=1
-    [ "$OK" = "1" ] &&  cool | tee -a ../linbox-auto-install.log  || die 
+    [ "$OK" = "1" ] &&  cool | tee -a ../linbox-auto-install.log  || die
 fi
 
 ### Fflas-ffpack ###
@@ -608,22 +608,22 @@ fi
 if [ -f Makefile ] ; then
     echo -e "${BEG}cleaning Givaro..."| tee -a ../../linbox-auto-install.log
     ${MAKEPROG} clean | tee -a ../../linbox-auto-install.log|| die
-    ${MAKEPROG} distclean | tee -a ../../linbox-auto-install.log|| die 
+    ${MAKEPROG} distclean | tee -a ../../linbox-auto-install.log|| die
 	# ${MAKEPROG} unistall || die
     cool
 fi
 
-echo -e "${BEG}configuring Givaro..."
+echo -e "${BEG}configuring Givaro..."| tee -a ../../linbox-auto-install.log
 
 if [ "$STABLE_VAR" = "true" ]; then
-    echo "./configure  $PREFIX $DEBUG $OPTIM $GMP $WARNINGS "
+    echo "./configure  $PREFIX $DEBUG $OPTIM $GMP $WARNINGS "| tee -a ../../linbox-auto-install.log
     echo "./configure  $PREFIX $DEBUG $OPTIM $GMP $WARNINGS " > configure.givaro.exe
     chmod +x configure.givaro.exe
     ./configure.givaro.exe | tee -a ../../linbox-auto-install.log
     rm -rf configure.givaro.exe
 	#./configure  $PREFIX $DEBUG $OPTIM $GMP $WARNINGS || die
 else
-    echo "./autogen.sh $PREFIX $DEBUG $OPTIM $GMP $WARNINGS"
+    echo "./autogen.sh $PREFIX $DEBUG $OPTIM $GMP $WARNINGS"| tee -a ../../linbox-auto-install.log
     echo "./autogen.sh $PREFIX $DEBUG $OPTIM $GMP $WARNINGS" > autogen.givaro.exe
     chmod +x autogen.givaro.exe
     ./autogen.givaro.exe| tee -a ../../linbox-auto-install.log
@@ -664,32 +664,32 @@ if [ "$OPENBLAS_VAR" = "true" ]; then
     else
 	cd OpenBLAS/ || die
     fi
-    
+
     if [ -f Makefile ] ; then
 	echo -e "${BEG}cleaning OpenBLAS..."| tee -a ../../linbox-auto-install.log
 	${MAKEPROG} clean | tee -a ../../linbox-auto-install.log|| die
-	${MAKEPROG} distclean | tee -a ../../linbox-auto-install.log|| die 
+	${MAKEPROG} distclean | tee -a ../../linbox-auto-install.log|| die
 	# ${MAKEPROG} unistall || die
 	cool
     fi
-    
+
     OPENBLAS_FLAGS="USE_THREADS=0 USE_THREAD=0 CC=gcc FC=gfortran PREFIX=$PREFIX_LOC"
-    
+
     echo -e "${BEG}building OpenBLAS..."| tee -a ../../linbox-auto-install.log
     echo "${MAKEPROG} ${OPENBLAS_FLAGS} CXXFLAGS+=\"$EXTRA\" LDFLAGS+=\"-Wl,-rpath,$PREFIX_LOC\""| tee -a ../../linbox-auto-install.log
-    
+
     if [ -n "$EXTRA" ] ; then
 	${MAKEPROG} ${OPENBLAS_FLAGS} "CXXFLAGS+=\"$EXTRA\" LDFLAGS+=\"-Wl,-rpath,$PREFIX_LOC\"" | tee -a ../../linbox-auto-install.log|| die
     else
 	${MAKEPROG} ${OPENBLAS_FLAGS} | tee -a ../../linbox-auto-install.log|| die
     fi
-    
+
     echo -e "${BEG}installing OpenBLAS..."| tee -a ../../linbox-auto-install.log
     ${MAKEPROG} ${OPENBLAS_FLAGS} install | tee -a ../../linbox-auto-install.log|| die
-    
+
 #return in build
     cd ..
-    
+
     cool| tee -a ../linbox-auto-install.log
 
     if [ "$BLAS_VAR" = "false" ]; then
@@ -719,9 +719,9 @@ if [ "$BLIS_VAR" = "true" ]; then
 	cool
     fi
 
-    echo -e "${BEG}configuring Blis..."
+    echo -e "${BEG}configuring Blis..."| tee -a ../../linbox-auto-install.log
 
-    echo "./configure  $PREFIX $DEBUG $WARNINGS --enable-cblas auto"
+    echo "./configure  $PREFIX $DEBUG $WARNINGS --enable-cblas auto"| tee -a ../../linbox-auto-install.log
     echo "./configure  $PREFIX $DEBUG $WARNINGS --enable-cblas auto " > configure.blis.exe
     chmod +x configure.blis.exe
     ./configure.blis.exe | tee -a ../../linbox-auto-install.log
@@ -766,7 +766,7 @@ fi
 if [ -f Makefile ] ; then
     echo -e "${BEG}cleaning Fflas-Ffpack..."| tee -a ../../linbox-auto-install.log
     ${MAKEPROG} clean | tee -a ../../linbox-auto-install.log|| die
-    ${MAKEPROG} distclean | tee -a ../../linbox-auto-install.log|| die 
+    ${MAKEPROG} distclean | tee -a ../../linbox-auto-install.log|| die
 	# ${MAKEPROG} unistall || die
     cool| tee -a ../../linbox-auto-install.log
 fi
@@ -790,7 +790,7 @@ else
 fi
 
 echo -e "${BEG}building Fflas-Ffpack..."| tee -a ../../linbox-auto-install.log
-echo "${MAKEPROG} CXXFLAGS+=\"$EXTRA\""
+echo "${MAKEPROG} CXXFLAGS+=\"$EXTRA\""| tee -a ../../linbox-auto-install.log
 if [ -n "$EXTRA" ] ; then
     ${MAKEPROG} "CXXFLAGS+=\"$EXTRA\"" | tee -a ../../linbox-auto-install.log|| die
 else
@@ -803,7 +803,7 @@ if [ "$CHECK_VAR" = "true" ] ; then
 fi
 
 
-echo -e "${BEG}installing Fflas-Ffpack..."
+echo -e "${BEG}installing Fflas-Ffpack..."| tee -a ../../linbox-auto-install.log
 ${MAKEPROG} install | tee -a ../../linbox-auto-install.log|| die
 
 cool| tee -a ../../linbox-auto-install.log
@@ -821,7 +821,7 @@ cd ..
 if [ -f Makefile ] ; then
     echo -e "${BEG}cleaning LinBox..."| tee -a ./linbox-auto-install.log
     ${MAKEPROG} clean | tee -a ./linbox-auto-install.log|| die
-    ${MAKEPROG} distclean | tee -a ./linbox-auto-install.log|| die 
+    ${MAKEPROG} distclean | tee -a ./linbox-auto-install.log|| die
 	# ${MAKEPROG} unistall || die
     cool| tee -a ./linbox-auto-install.log
 fi
@@ -834,7 +834,7 @@ echo -e " *   'export LD_LIBRARY_PATH=\$LD_LIBRARY_PATH:$PREFIX_LOC/lib'"| tee -
 echo -e " * to ensure you don't get undefined symbols !"| tee -a ./linbox-auto-install.log
 echo  ""| tee -a ./linbox-auto-install.log
 
-if [ -x autogen.sh ] ;  then 
+if [ -x autogen.sh ] ;  then
     echo "./autogen.sh $PREFIX $DEBUG $OPTIM $GMP $BLAS $NTL $WARNINGS $IML $DRIV"| tee -a ./linbox-auto-install.log
     ./autogen.sh "$PREFIX" "$DEBUG" "$OPTIM" "$GMP" "$BLAS" "$NTL" "$WARNINGS" "$IML" "$DRIV" | tee -a ./linbox-auto-install.log|| die
 else
