@@ -1,4 +1,3 @@
-
 /* linbox/algorithms/block-massey-domain.h
  * Copyright (C) 2002 Pascal Giorgi
  *
@@ -30,36 +29,61 @@
 
 #include <vector>
 #include <iostream>
+#include <algorithm>
 #include <iomanip>
 
-
 #include "linbox/util/commentator.h"
 #include "linbox/util/timer.h"
-#include "linbox/field/unparametric.h"
+#include <givaro/zring.h>
 #include "linbox/matrix/matrix-domain.h"
-#include "linbox/matrix/blas-matrix.h"
+#include "linbox/matrix/dense-matrix.h"
 #include "linbox/matrix/factorized-matrix.h"
-#include "linbox/algorithms/blas-domain.h"
-#include "linbox/algorithms/sigma-basis.h"
-
+#include "linbox/matrix/matrix-domain.h"
+#include "linbox/algorithms/polynomial-matrix/order-basis.h"
+#include "linbox/matrix/polynomial-matrix.h"
 
 #include "linbox/util/timer.h"
 
-//#define  __CHECK_RESULT
-//#define __DEBUG_MAPLE
-//#define __CHECK_LOOP
-//#define __PRINT_MINPOLY
-//#define __CHECK_DISCREPANCY
-//#define __CHECK_TRANSFORMATION
-//#define __CHECK_SIGMA_RESULT
-//#define __PRINT_SEQUENCE
-#define _BM_TIMING
+// #define  __CHECK_RESULT
+// #define __DEBUG_MAPLE
+// #define __CHECK_LOOP
+// #define __CHECK_DISCREPANCY
+// #define __CHECK_TRANSFORMATION
+//  #define __CHECK_SIGMA_RESULT
+//  #define __PRINT_SEQUENCE
+//  #define __PRINT_SIGMABASE
+// #define __PRINT_MINPOLY
+
+//#define _BM_TIMING
+#define DEFAULT_BLOCK_EARLY_TERM_THRESHOLD 10
 
 namespace LinBox
 {
+	template<class Field, class Coefficient>
+		void write_maple(const Field& F, const std::vector<Coefficient> & P)
+		{
+			std::cout<<"Matrix([";
+			for (size_t i=0;i< P[0].rowdim();++i){
+				std::cout<<"[";
+				for (size_t j=0;j< P[0].coldim();++j){
+					F.write(std::cout,P[0].getEntry(i,j));
+					for (size_t k=1;k<P.size();++k){
+						std::cout<<"+ x^"<<k<<"*";
+						F.write(std::cout,P[k].getEntry(i,j));
+					}
+					if (j != P[0].coldim()-1)
+						std::cout<<",";
+				}
+				if (i != P[0].rowdim()-1)
+					std::cout<<"],";
+				else
+					std::cout<<"]";
+			}
+			std::cout<<"]);\n";
+		}
+
 
 
-#define DEFAULT_EARLY_TERM_THRESHOLD 20
 
 
 	/** Compute the linear generator of a sequence of matrices.
@@ -67,7 +91,7 @@ namespace LinBox
 	 * This class encapsulates the functionality required for computing
 	 * the block minimal polynomial of a matrix.
 	 * @bib
-	 * Giorgi, Jeannerod Villard algorithm from ISSAC'03
+	 * Giorgi, Jeannerod, Villard algorithm from ISSAC'03
 	 */
 	template<class _Field, class _Sequence>
 	class BlockMasseyDomain {
@@ -76,15 +100,16 @@ namespace LinBox
 		typedef _Field                           Field;
 		typedef typename Field::Element        Element;
 		typedef _Sequence                     Sequence;
-		typedef BlasMatrix<Field>        Coefficient;
+		typedef BlasMatrix<Field>          Coefficient;
+                typedef BlasSubmatrix<Coefficient>   CoeffView;
 
 
 	private:
 		Sequence                          *_container;
-		Field                                      _field;
+		const Field                           *_field;
 		BlasMatrixDomain<Field>                  _BMD;
 		MatrixDomain<Field>                       _MD;
-		unsigned long            EARLY_TERM_THRESHOLD;
+		size_t            EARLY_TERM_THRESHOLD;
 
 
 	public:
@@ -97,7 +122,7 @@ namespace LinBox
 		mutable Timer   ttUpdateSigma;			mutable Timer tUpdateSigma;
 		mutable Timer   ttInverseL;			mutable Timer tInverseL;
 		mutable Timer   ttGetPermutation;		mutable Timer tGetPermutation;
-		mutable Timer   ttLQUP;				mutable Timer tLQUP;
+		mutable Timer   ttPLUQ;				mutable Timer tPLUQ;
 		mutable Timer   ttDiscrepancy;			mutable Timer tDiscrepancy;
 		mutable Timer   ttGetCoeff;			mutable Timer tGetCoeff;
 		mutable Timer   ttCheckSequence;		mutable Timer tCheckSequence;
@@ -117,7 +142,7 @@ namespace LinBox
 			ttUpdateSigma.clear();
 			ttInverseL.clear();
 			ttGetPermutation.clear();
-			ttLQUP.clear();
+			ttPLUQ.clear();
 			ttDiscrepancy.clear();
 			ttGetCoeff.clear();
 			ttCheckSequence.clear();
@@ -147,7 +172,7 @@ namespace LinBox
 			print(ttCheckSequence, "Rank of Seq[0]", "direct");
 			print(ttGetCoeff, "Compute sequence", "direct");
 			print(ttDiscrepancy, "Compute Discrepancy", "direct");
-			print(ttLQUP, "LQUP","direct");
+			print(ttPLUQ, "PLUQ","direct");
 			print(ttGetPermutation, "Compute Permutation", "direct");
 			print(ttApplyPerm, "Apply Permutation", "direct");
 			print(ttInverseL, "Inverse of L", "direct");
@@ -164,17 +189,18 @@ namespace LinBox
 #endif
 
 
-		BlockMasseyDomain (const BlockMasseyDomain<Field, Sequence> &Mat, unsigned long ett_default = DEFAULT_EARLY_TERM_THRESHOLD) :
-			_container(Mat._container), _field(Mat._field), _BMD(Mat._field),
-			_MD(Mat._field),  EARLY_TERM_THRESHOLD (ett_default)
+		BlockMasseyDomain (const BlockMasseyDomain<Field, Sequence> &Mat, size_t ett_default = DEFAULT_BLOCK_EARLY_TERM_THRESHOLD) :
+			_container(Mat._container), _field(Mat._field), _BMD(Mat.field()),
+			_MD(Mat.field()),  EARLY_TERM_THRESHOLD (ett_default)
 		{
 #ifdef _BM_TIMING
 			clearTimer();
 #endif
+
 		}
 
-		BlockMasseyDomain (Sequence *D, unsigned long ett_default = DEFAULT_EARLY_TERM_THRESHOLD) :
-			_container(D), _field(D->getField ()), _BMD(D->getField ()), _MD(D->getField ()), EARLY_TERM_THRESHOLD (ett_default)
+		BlockMasseyDomain (Sequence *D, size_t ett_default = DEFAULT_BLOCK_EARLY_TERM_THRESHOLD) :
+			_container(D), _field(&(D->field ())), _BMD(D->field ()), _MD(D->field ()), EARLY_TERM_THRESHOLD (ett_default)
 		{
 #ifdef _BM_TIMING
 			clearTimer();
@@ -183,8 +209,8 @@ namespace LinBox
 
 
 		// field of the domain
-		const Field &getField    () const
-		{ return _field; }
+		const Field &field    () const
+		{ return *_field; }
 
 		// sequence of the domain
 		Sequence *getSequence () const
@@ -220,37 +246,12 @@ namespace LinBox
 
 	private:
 
-		template<class Field>
-		void write_maple(const Field& F, const std::vector<Coefficient> & P)
-		{
-			std::cout<<"Matrix([";
-			for (size_t i=0;i< P[0].rowdim();++i){
-				std::cout<<"[";
-				for (size_t j=0;j< P[0].coldim();++j){
-					F.write(std::cout,P[0].getEntry(i,j));
-					for (size_t k=1;k<P.size();++k){
-						std::cout<<"+ x^"<<k<<"*";
-						F.write(std::cout,P[k].getEntry(i,j));
-					}
-					if (j != P[0].coldim()-1)
-						std::cout<<",";
-				}
-				if (i != P[0].rowdim()-1)
-					std::cout<<"],";
-				else
-					std::cout<<"]";
-			}
-			std::cout<<"]);\n";
-		}
 
 
 		std::vector<size_t> masseyblock_left (std::vector<Coefficient> &P)
 		{
+                        std::ostream& report = commentator().report (Commentator::LEVEL_IMPORTANT, INTERNAL_DESCRIPTION);
 
-#ifdef _BM_TIMING
-			tSetup.clear();
-			tSetup.start();
-#endif
 			const size_t length = _container->size ();
 			const size_t m = _container->rowdim();
 			const size_t n = _container->coldim();
@@ -263,23 +264,21 @@ namespace LinBox
 			typename Sequence::const_iterator _iter (_container->begin ());
 
 			// Reservation of memory for the entire sequence
-			std::vector<Coefficient> S (length,Coefficient(m,n));
-			//std::vector<Coefficient> S (length); //,Coefficient(m,n));
+                        const Coefficient Zeromn(field(),m,n);
+			std::vector<Coefficient> S (length,Zeromn);
+
+
+			Coefficient Unit(field(),m+n,m);
+			const Coefficient Zero(field(),m+n,m);
 
-			Coefficient Unit(m+n,m);
-			const Coefficient Zero(m+n,m);
-			Element one,zero,mOne;
-			_field.init(one,1L);
-			_field.init(zero,0L);
-			_field.init(mOne,-1L);
 			for (size_t i=0;i<m;i++)
-				Unit.setEntry(i,i,one);
+				Unit.setEntry(i,i,field().one);
 			size_t min_mn=(m <n)? m :n;
 
 			// initialization of discrepancy
-			Coefficient Discrepancy(m+n,n);
+			Coefficient Discrepancy(field(),m+n,n);
 			for (size_t i=0;i<n;i++)
-				Discrepancy.setEntry(i+m,i,one);
+				Discrepancy.setEntry(i+m,i,field().one);
 
 			// initialization of sigma base
 			std::vector<Coefficient> SigmaBase(1, Unit);
@@ -293,75 +292,58 @@ namespace LinBox
 			std::vector<long> degree(m+n,0);
 			for (size_t i=0;i<m;++i)
 				degree[i]=0;
-#ifdef _BM_TIMING
-			tSetup.stop();
-			ttSetup += tSetup;
-			tCheckSequence.clear();
-			tCheckSequence.start();
-#endif
+
 			// The first sequence element should be of full rank
 			// this is due to the strategy which say that we can compute
 			// only the first column of the approximation of [ S(x) Id]^T
 			// since the other colums have always lower degree.
 			if (_BMD.rank(*_iter)< min_mn)
 				throw PreconditionFailed (__func__, __LINE__, "Bad random Blocks, abort\n");
-#ifdef _BM_TIMING
-			tCheckSequence.stop();
-			ttCheckSequence += tCheckSequence;
-#endif
 
-			unsigned long early_stop=0;
+			size_t early_stop=0;
 			long NN;
 			for (NN = 0; (NN < (long)length) && (early_stop < EARLY_TERM_THRESHOLD) ; ++NN, ++_iter) {
 
 				// Get the next coefficient in the sequence
 				S[NN]=*_iter;
-#ifdef  _BM_TIMING
-				if (NN != 0){
-					tGetCoeff.stop();
-					ttGetCoeff += tGetCoeff;
-				}
-				tDiscrepancy.clear();
-				tDiscrepancy.start();
-#endif
 
 				/*
 				 * Compute the new discrepancy (just updating the first m rows)
 				 */
 				// view of m first rows of SigmaBasis[0]
-				Coefficient Sigma(SigmaBase[0],0,0,m,m);
+				CoeffView Sigma(SigmaBase[0],0,0,m,m);
 
 				// view of m first rows of Discrepancy
-				Coefficient Discr(Discrepancy,0,0,m,n);
+				CoeffView Discr(Discrepancy,0,0,m,n);
 
 				_BMD.mul(Discr,Sigma,S[NN]);
+
 				for (size_t i=1;i<SigmaBase.size();i++){
-					Coefficient  Sigmaview(SigmaBase[i],0,0,m,m);
+					CoeffView  Sigmaview(SigmaBase[i],0,0,m,m);
 					_BMD.axpyin(Discr,Sigmaview,S[NN-i]);
 				}
 
-#ifdef _BM_TIMING
-				tDiscrepancy.stop();
-				ttDiscrepancy += tDiscrepancy;
-#endif
-
-				typename Coefficient::Iterator _iter_Discr = Discr.Begin();
-
-				while ((_field.isZero(*_iter_Discr) && _iter_Discr != Discr.End()))
+				typename CoeffView::Iterator _iter_Discr = Discr.Begin();
+				while (_iter_Discr != Discr.End() && (field().isZero(*_iter_Discr)))
 					++_iter_Discr;
+                                if (_iter_Discr!=Discr.End())
+                                        early_stop=0;
+                                else
+                                        early_stop++;
 
 				// maybe there is something to do here
 				// increase the last n rows of orders
 				// multiply by X the last n rows of SigmaBase
-				if (_iter_Discr != Discr.End())
-					early_stop=0;
-				else {
-					early_stop++;
-				}
-#ifdef _BM_TIMING
-				tGetPermutation.clear();
-				tGetPermutation.start();
-#endif
+				//if (_iter_Discr != Discr.End())
+
+                                /*
+                                Coefficient ZeroD(field(),m+n,n);
+                                if (_MD.areEqual(Discrepancy,ZeroD)) 
+                                        early_stop=0;
+                                else
+                                        early_stop++;
+                                */
+
 				// Computation of the permutation BPerm1 such that BPerm1.order is in increasing order.
 				// order=Perm.order
 				//! @todo factorize this in \c BlasPermutation.
@@ -380,56 +362,30 @@ namespace LinBox
 				}
 				BlasPermutation<size_t> BPerm1(Perm1);
 
-#ifdef _BM_TIMING
-				tGetPermutation.stop();
-				ttGetPermutation += tGetPermutation;
-				tApplyPerm.clear();
-				tApplyPerm.start();
-
-#endif
 				// Discrepancy= BPerm1.Discrepancy
 				_BMD.mulin_right(BPerm1,Discrepancy);
 
-#ifdef _BM_TIMING
-				tApplyPerm.stop();
-				ttApplyPerm += tApplyPerm;
-				tLQUP.clear();
-				tLQUP.start();
-#endif
-
 
 #ifdef __CHECK_DISCREPANCY
-				std::ostream& report = commentator().report (Commentator::LEVEL_IMPORTANT, INTERNAL_DESCRIPTION);
+
 				report<<"Discrepancy"<<NN<<":=Matrix(";
-				Discrepancy.write(report,_field,true)<<");"<<std::endl;
+				Discrepancy.write(report)<<");"<<std::endl;
 #endif
 
-
-
-				// Computation of the LQUP decomposition of the discrepancy
-				Coefficient CopyDiscr;
-				CopyDiscr=Discrepancy;
+				// Computation of the PLUQ decomposition of the discrepancy
+				Coefficient CopyDiscr(Discrepancy);
 				BlasPermutation<size_t> Pp (CopyDiscr.coldim());
 				BlasPermutation<size_t> Qt (CopyDiscr.rowdim());
-				LQUPMatrix<Field> LQUP(_field, CopyDiscr,Pp,Qt);
+				PLUQMatrix<Field> PLUQ(CopyDiscr,Pp,Qt);
 
-#ifdef _BM_TIMING
-				tLQUP.stop();
-				ttLQUP += tLQUP;
+				// Get the matrix L of PLUQ decomposition
+				TriangularBlasMatrix<Field> L(field(),m+n,m+n, Tag::Shape::Lower, Tag::Diag::Unit );
+				PLUQ.getL(L);
 
-#endif
-				// Get the matrix L of LQUP decomposition
-				TriangularBlasMatrix<Field> L(_field,m+n,m+n, LinBoxTag::Lower, LinBoxTag::Unit );
-				LQUP.getL(L);
-
-				// Get the tranposed  permutation of Q from LQUP
-				// BlasPermutation<size_t> Qt=LQUP.getQ();
+				// Get the tranposed  permutation of Q from PLUQ
+				// BlasPermutation<size_t> Qt=PLUQ.getQ();
 
 
-#ifdef _BM_TIMING
-				tGetPermutation.clear();
-				tGetPermutation.start();
-#endif
 				// Computation of permutations BPerm2 such that the last n rows of BPerm2.Qt.Discrepancy are non zero.
 				std::vector<size_t> Perm2(m+n);
 				for (size_t i=0;i<n;++i)
@@ -438,65 +394,27 @@ namespace LinBox
 					Perm2[i]=i;
 				BlasPermutation<size_t> BPerm2(Perm2);
 
-#ifdef _BM_TIMING
-				tGetPermutation.stop();
-				ttGetPermutation += tGetPermutation;
-				tInverseL.clear();
-				tInverseL.start();
-#endif
 				// compute the inverse of L
-				TriangularBlasMatrix<Field> invL (_field,m+n,m+n, LinBoxTag::Lower,LinBoxTag::Unit);
-				FFPACK::trinv_left((typename Field::Father_t)_field,m+n,L.getPointer(),L.getStride(),invL.getWritePointer(),invL.getStride());
-
-#ifdef _BM_TIMING
-				tInverseL.stop();
-				ttInverseL += tInverseL;
-#endif
+				TriangularBlasMatrix<Field> invL (field(),m+n,m+n, Tag::Shape::Lower,Tag::Diag::Unit);
+				FFPACK::trinv_left(field(),m+n,L.getPointer(),L.getStride(),invL.getWritePointer(),invL.getStride());
 
 #ifdef 	__CHECK_TRANSFORMATION
-				report<<"invL"<<N<<":=Matrix(";
-				invL.write(report,_field,true)<<");"<<std::endl;
+				report<<"invL"<<NN<<":=Matrix(";
+				invL.write(report)<<");"<<std::endl;
 
 #endif
 				// SigmaBase =  BPerm2.Qt. L^(-1) . BPerm1 . SigmaBase
 				for (size_t i=0;i<SigmaBase.size();i++) {
-#ifdef _BM_TIMING
-					tApplyPerm.clear();
-					tApplyPerm.start();
-#endif
 					_BMD.mulin_right(BPerm1,SigmaBase[i]);
-
-#ifdef _BM_TIMING
-					tApplyPerm.stop();
-					ttApplyPerm +=tApplyPerm;
-
-					tUpdateSigma.clear();
-					tUpdateSigma.start();
-#endif
 					_BMD.mulin_right(invL,SigmaBase[i]);
-#ifdef _BM_TIMING
-					tUpdateSigma.stop();
-					ttUpdateSigma += tUpdateSigma;
-					tApplyPerm.clear();
-					tApplyPerm.start();
-#endif
 					_BMD.mulin_right(Qt,SigmaBase[i]);
 					_BMD.mulin_right(BPerm2,SigmaBase[i]);
-#ifdef _BM_TIMING
-					tApplyPerm.stop();
-					ttApplyPerm +=tApplyPerm;
-#endif
 				}
 
-
-#ifdef _BM_TIMING
-				tApplyPerm.clear();
-				tApplyPerm.start();
-#endif
-
 				// Apply BPerm2 and Qt to the vector of order and increase by 1 the last n rows
-				UnparametricField<long> UF(0);
-				BlasMatrixDomain<UnparametricField<long> > BMDUF(UF);
+				Givaro::ZRing<long> UF(0);
+				// What?
+				BlasMatrixDomain<Givaro::ZRing<long> > BMDUF(UF);
 				BMDUF.mulin_right(Qt,order);
 				BMDUF.mulin_right(BPerm2,order);
 				BMDUF.mulin_right(BPerm1,degree);
@@ -507,12 +425,6 @@ namespace LinBox
 					degree[i]++;
 				}
 
-#ifdef _BM_TIMING
-				tApplyPerm.stop();
-				ttApplyPerm += tApplyPerm;
-				tShiftSigma.clear();
-				tShiftSigma.start();
-#endif
 				// Multiplying the last n row of SigmaBase by x.
 				long max_degree=degree[m];
 				for (size_t i=m+1;i<m+n;++i) {
@@ -530,34 +442,17 @@ namespace LinBox
 				for (int i= (int)size-2;i>=0;i--)
 					for (size_t j=0;j<n;j++)
 						for (size_t k=0;k<n;++k){
-
-							// report << " i+1 item: ";
-							// report << SigmaBase[i+1].getEntry(m+j,k) ;
-							// report << " i item: ";
-					 		// report << SigmaBase[i].getEntry(m+j,k)
-							// << std::endl;
-							// typename Field::Element& x = SigmaBase[i+1].refEntry(m+j,k);
-							// report << &x << " " << x << " &x and x" << std::endl;
-							// x = SigmaBase[i].getEntry(m+j,k);
-							// report << x << " new x" << std::endl;
-							_field.assign(SigmaBase[i+1].refEntry(m+j,k), SigmaBase[i].getEntry(m+j,k));
+							field().assign(SigmaBase[i+1].refEntry(m+j,k), SigmaBase[i].getEntry(m+j,k));
 
 						}
 
 				for (size_t j=0;j<n;j++)
 					for (size_t k=0;k<n;++k)
-						_field.assign(SigmaBase[0].refEntry(m+j,k),zero);
-
-
-#ifdef _BM_TIMING
-				tShiftSigma.stop();
-				ttShiftSigma += tShiftSigma;
-#endif
-
+						field().assign(SigmaBase[0].refEntry(m+j,k),field().zero);
 
 #ifdef __DEBUG_MAPLE
 				report<<"\n\nSigmaBase"<<NN<<":= ";
-				write_maple(_field,SigmaBase);
+				write_maple(field(),SigmaBase);
 
 				report<<"order"<<NN<<":=<";
 				for (size_t i=0;i<m+n;++i){
@@ -580,46 +475,29 @@ namespace LinBox
 				report<<"Sequence size:  "<<NN+1<<std::endl;
 				size_t min_t = (SigmaBase.size() > NN+1)? NN+1: SigmaBase.size();
 				for (size_t i=min_t - 1 ; i<NN+1; ++i){
-					Coefficient Disc(m+n,n);
-					_BMD.mul(Disc,SigmaBase[0],S[i]);
-					for (size_t j=1;j<min_t -1;++j)
+					Coefficient Disc(field(),m+n,n);
+					for (size_t j=0;j<min_t ;++j)
 						_BMD.axpyin(Disc,SigmaBase[j],S[i-j]);
-					Disc.write(report,_field)<<std::endl;
+					Disc.write(report)<<std::endl;
 				}
 #endif
 
-
-#ifdef _BM_TIMING
-				tNewDiscrepancy.clear();
-				tNewDiscrepancy.start();
-#endif
-				// Discrepancy= BPerm2.U.Pp from LQUP
-				Coefficient U(m+n,n);
-				TriangularBlasMatrix<Field> trU(U,LinBoxTag::Upper,LinBoxTag::NonUnit);
-				LQUP.getU(trU);
+				// Discrepancy= BPerm2.U.Pp from PLUQ
+				Coefficient U(field(),m+n,n);
+				TriangularBlasMatrix<Field> trU(U,Tag::Shape::Upper,Tag::Diag::NonUnit);
+				PLUQ.getU(trU);
 				//Discrepancy=U;
-				// BlasPermutation<size_t> Pp= LQUP.getP();
+				// BlasPermutation<size_t> Pp= PLUQ.getP();
 				_BMD.mul(Discrepancy,trU, Pp);
 				_BMD.mulin_right(BPerm2,Discrepancy);
-
-#ifdef _BM_TIMING
-				tNewDiscrepancy.stop();
-				ttNewDiscrepancy+=tNewDiscrepancy;
-
-				// timer in the loop
-				tGetCoeff.clear();
-				tGetCoeff.start();
-#endif
-
 			}
-			std::ostream& report = commentator().report (Commentator::LEVEL_IMPORTANT, INTERNAL_DESCRIPTION);
 
-			if ( early_stop == EARLY_TERM_THRESHOLD)
+                        if ( early_stop == EARLY_TERM_THRESHOLD)
 				report<<"Early termination is used: stop at "<<NN<<" from "<<length<<" iterations\n\n";
 
 #ifdef __PRINT_SEQUENCE
 			report<<"\n\nSequence:= ";
-			write_maple(_field,S);
+			write_maple(field(),S);
 #endif
 
 
@@ -627,54 +505,35 @@ namespace LinBox
 #ifdef __CHECK_SIGMA_RESULT
 			report<<"Check SigmaBase application\n";
 			for (size_t i=SigmaBase.size()-1 ;i< length ;++i){
-				Coefficient res(m+n,n);
+				Coefficient res(field(),m+n,n);
 				for (size_t k=0;k<SigmaBase.size();++k)
 					_BMD.axpyin(res,SigmaBase[k],S[i-k]);
-				res.write(report,_field)<<std::endl;
+				res.write(report)<<std::endl;
 			}
 
 #endif
 
-#ifdef _BM_TIMING
-			tGetMinPoly.clear();
-			tGetMinPoly.start();
-#endif
 			// Get the reverse matrix polynomial of the first m rows of SigmaBase according to degree.
 			degree=order;
-			long max=degree[0];
-			for (size_t i=1;i<m;i++) {
-				if (degree[i]>max)
-					max=degree[i];
-			}
-			//P = std::vector<Coefficient> (max+1);
-			P.clear();
-			Coefficient tmp(m,m);
-			P.resize(max+1, tmp);
-			//for (long i=0;i< max+1;++i)
-			//	P[i]=tmp;
+			long max= *std::max_element(degree.begin(),degree.end());
 
+			Coefficient tmp(field(),m,m);
+                        P = std::vector<Coefficient> (max+1,tmp);
 			for (size_t i=0;i<m;i++)
 				for (long j=0;j<=degree[i];j++)
 					for (size_t k=0;k<m;k++)
-						_field.assign(P[degree[i]-j].refEntry(i,k), SigmaBase[j].getEntry(i,k));
-#ifdef _BM_TIMING
-			tGetMinPoly.stop();
-			ttGetMinPoly +=tGetMinPoly;
-#endif
-
-
+						field().assign(P[degree[i]-j].refEntry(i,k), SigmaBase[j].getEntry(i,k));
 #ifdef __CHECK_RESULT
 			report<<"Check minimal polynomial application\n";
 			bool valid=true;
 			for (size_t i=0;i< NN - P.size();++i){
-				Coefficient res(m,n);
+				Coefficient res(field(),m,n);
 				_BMD.mul(res,P[0],S[i]);
 				for (size_t k=1,j=i+1;k<P.size();++k,++j)
 					_BMD.axpyin(res,P[k],S[j]);
 				for (size_t j=0;j<m*n;++j)
-					if (!_field.isZero(*(res.getPointer()+j)))
+					if (!field().isZero(*(res.getPointer()+j)))
 						valid= false;
-				//res.write(report,_field)<<std::endl;
 			}
 			if (valid)
 				report<<"minpoly is correct\n";
@@ -684,136 +543,127 @@ namespace LinBox
 
 #ifdef __PRINT_MINPOLY
 			report<<"MinPoly:=";
-			write_maple(_field,P);
-#if 0
+			write_maple(field(),P);
 			Coefficient Mat(*_container->getBB());
 			report<<"A:=Matrix(";
-			Mat.write(report,_field,true);
-#endif
+			Mat.write(report);
 #endif
 
 			std::vector<size_t> deg(m);
 			for (size_t i=0;i<m;++i)
 				deg[i]=(size_t)degree[i];
 
-			//report << "clearing S " << S.size() << std::endl;
-			//S.clear();
-			//report << "cleared S " << S.size() << std::endl;
-			// report << "clearing SigmaBase " << SigmaBase.size() << std::endl;
-			// SigmaBase.resize(SigmaBase.size()-2);
-			// report << "clearing last 4 of SigmaBase " << SigmaBase.size() << std::endl;
-			// SigmaBase.clear();
-			// report << "cleared SigmaBase " << SigmaBase.size() << std::endl;
 			return deg;
 		}
 
 
-		std::vector<size_t> masseyblock_left_rec (std::vector<Coefficient> &P)
+		std::vector<size_t> masseyblock_left_rec (std::vector<Coefficient> &lingen)
 		{
 #ifdef __CHECK_RESULT
-			std::ostream& report = commentator().report (Commentator::LEVEL_IMPORTANT, INTERNAL_DESCRIPTION);
+			//std::ostream& report = commentator().report (Commentator::LEVEL_IMPORTANT, INTERNAL_DESCRIPTION);
+                        std::ostream& report = std::cout;
 #endif
 			// Get information of the Sequence (U.A^i.V)
 			size_t length = _container->size();
-			size_t m, n;
+			size_t m, n, mn;
 			m = _container->rowdim();
 			n = _container->coldim();
-
-			// Set some useful constant
-			Element one;
-			_field.init(one,1UL);
-			const Coefficient Zero(2*m,2*m);
-
+                        mn=m+n;
+                        
+                        
 			// Make the Power Serie from  Sequence (U.A^i.V) and Identity
-			//_container->recompute(); // make sure sequence is already computed
-			std::vector<Coefficient> PowerSerie(length);
+			//_container->recompute(); // make sure sequence is already computed			
+                        typedef PolynomialMatrix<PMType::polfirst,PMStorage::plain, Field> PMatrix;
+                        PMatrix PowerSerie(field(),mn,n,length);
+                        
 			typename Sequence::const_iterator _iter (_container->begin ());
-			for (size_t i=0;i< length; ++i, ++_iter){
-				Coefficient value(2*m,n);
-				PowerSerie[i] = value;
+			for (size_t i=0;i< length; ++i, ++_iter)
 				for (size_t j=0;j<m;++j)
 					for (size_t k=0;k<n;++k)
-						PowerSerie[i].setEntry(j,k, (*_iter).getEntry(j,k));
-			}
+						field().assign(PowerSerie.ref(j,k,i), (*_iter).getEntry(j,k));                        
 			for (size_t j=0;j<n;++j)
-				PowerSerie[0].setEntry(m+j, j, one);
+				field().assign(PowerSerie.ref(m+j,j,0),field().one);
 #ifdef __PRINT_SEQUENCE
 			report<<"PowerSerie:=";
-			write_maple(_field,PowerSerie);
+			PowerSerie.write(report);
 #endif
 
-
-			// Set the defect to [0 ... 0 1 ... 1]^T
-			std::vector<size_t> defect(2*m,0);
-			for (size_t i=m;i< 2*m;++i)
-				defect[i]=1;
+                        // set the shift to [ 0 .. 0 1 .. 1]
+                        std::vector<size_t> shift(mn,0);
+                        std::fill(shift.begin()+m,shift.end(),1);
 
 			// Prepare SigmaBase
-			std::vector<Coefficient> SigmaBase(length,Zero);
+			PMatrix SigmaBase(field(),mn,mn,length);
+
+			// Compute OrderBasis up to the order length 
+                        OrderBasis<Field> SB(field());
+                        SB.PM_Basis(SigmaBase, PowerSerie, length, shift);
 
-			// Compute Sigma Base up to the order length - 1
-			SigmaBasis<Field> SB(_field, PowerSerie);
-			SB.left_basis(SigmaBase, length-1, defect);
 
 			// take the m rows which have lowest defect
 			// compute permutation such that first m rows have lowest defect
-			std::vector<size_t> Perm(2*m);
-			for (size_t i=0;i<2*m;++i)
+			std::vector<size_t> Perm(mn);
+			for (size_t i=0;i<mn;++i)
 				Perm[i]=i;
-			for (size_t i=0;i<2*m;++i) {
+			for (size_t i=0;i<mn;++i) {
 				size_t idx_min=i;
-				for (size_t j=i+1;j<2*m;++j)
-					if (defect[j]< defect[idx_min])
+				for (size_t j=i+1;j<m+n;++j)
+					if (shift[j]< shift[idx_min])
 						idx_min=j;
-				std::swap(defect[i],defect[idx_min]);
+				std::swap(shift[i],shift[idx_min]);
 				Perm[i]=idx_min;
 			}
-			BlasPermutation<size_t> BPerm(Perm);
+
+                        // convert to polynomial of matrices
+                        PolynomialMatrix<PMType::matfirst,PMStorage::plain, Field> Sigma (field(), mn,mn,SigmaBase.size());                        
+                        Sigma.copy(SigmaBase);
+
+                        BlasPermutation<size_t> BPerm(Perm);
 
 			// Apply BPerm to the Sigma Base
-			for (size_t i=0;i<SigmaBase.size();++i)
-				_BMD.mulin_right(BPerm,SigmaBase[i]);
+			for (size_t i=0;i<Sigma.size();++i)
+				_BMD.mulin_right(BPerm,Sigma[i]);
 
-#if 0
+
+                        
+#ifdef __PRINT_SIGMABASE
+                        report<<"order is "<<length-1<<std::endl;
 			report<<"SigmaBase:=";
-			write_maple(_field,SigmaBase);
+                        Sigma.write(report);
+                        report<<"shift:=[";
+                        std::ostream_iterator<int> out_it (report,", ");
+                        std::copy ( shift.begin(), shift.end(), out_it );
+                        report<<"];\n";
+                        
 #endif
-			// Compute the reverse polynomial of SigmaBase according to defect of each row
-			size_t max=defect[0];
-			for (size_t i=0;i<m;++i)
-				if (defect[i] > max)
-					max=defect[i];
 
-			P = std::vector<Coefficient> (max+1);
-			Coefficient tmp(m,m);
-			for (size_t i=0;i< max+1;++i)
-				P[i]=tmp;
-			for (size_t i=0;i<m;i++)
-				for (size_t j=0;j<=defect[i];j++)
-					for (size_t k=0;k<m;k++)
-						_field.assign(P[defect[i]-j].refEntry(i,k), SigmaBase[j].getEntry(i,k));
+                        // Compute the reverse polynomial of Sigma according to row shift 
+                        size_t max= *std::max_element(shift.begin(),shift.begin()+m);
+                        //PMatrix lingen(field(),m,m,max+1);
+                        Coefficient Zeromm(field(),m,m);
+                        lingen.resize(max+1,Zeromm);
+                        for (size_t i=0;i<m;i++)
+                                for (size_t j=0;j<=shift[i];j++)
+                                        for (size_t k=0;k<m;k++)
+                                                field().assign(lingen[shift[i]-j].refEntry(i,k), Sigma.ref(i,k,j));
 
+
+                     
 #ifdef __CHECK_RESULT
 			report<<"Check minimal polynomial application\n";
-			//_container->recompute();
-			typename Sequence::const_iterator _ptr (_container->begin ());
-			for (size_t i=0;i< length; ++i, ++_ptr){
-				PowerSerie[i] = *_ptr;
-			}
 			bool valid=true;
-			for (size_t i=0;i< length - P.size();++i){
-				Coefficient res(m,n);
+			for (size_t i=0;i< length - lingen.size();++i){
+				Coefficient res(field(),m,n);
 				Coefficient Power(PowerSerie[i],0,0,m,n);
-				_BMD.mul(res,P[0],Power);
-				for (size_t k=1,j=i+1;k<P.size();++k,++j){
+				_BMD.mul(res,lingen[0],Power);
+				for (size_t k=1,j=i+1;k<lingen.size();++k,++j){
 					Coefficient Powerview(PowerSerie[j],0,0,m,n);
-					_BMD.axpyin(res,P[k],Powerview);
+					_BMD.axpyin(res,lingen[k],Powerview);
 				}
 				for (size_t j=0;j<m*n;++j)
-					if (!_field.isZero(*(res.getPointer()+j)))
+					if (!field().isZero(*(res.getPointer()+j)))
 						valid= false;
-				//res.write(report,_field)<<std::endl;
-			}
+                        }
 			if (valid)
 				report<<"minpoly is correct\n";
 			else
@@ -822,14 +672,11 @@ namespace LinBox
 
 #ifdef __PRINT_MINPOLY
 			report<<"MinPoly:=";
-			write_maple(_field,P);
-			//Coefficient Mat(*_container->getBB());
-			//report<<"A:=Matrix(";
-			//Mat.write(report,_field,true);
+                        write_maple(field(),lingen);
 #endif
 			std::vector<size_t> degree(m);
 			for (size_t i=0;i<m;++i)
-				degree[i] = defect[i];
+				degree[i] = shift[i];
 			return degree;
 		}
 
@@ -839,11 +686,10 @@ namespace LinBox
 
 #endif // __LINBOX_massey_block_domain_H
 
-// vim:sts=8:sw=8:ts=8:noet:sr:cino=>s,f0,{0,g0,(0,:0,t0,+0,=s
 // Local Variables:
 // mode: C++
-// tab-width: 8
+// tab-width: 4
 // indent-tabs-mode: nil
-// c-basic-offset: 8
+// c-basic-offset: 4
 // End:
-
+// vim:sts=4:sw=4:ts=4:et:sr:cino=>s,f0,{0,g0,(0,\:0,t0,+0,=s
