@@ -30,12 +30,12 @@
 #include "fflas-ffpack/ffpack/ffpack.h"
 #include "linbox/algorithms/matrix-hom.h"
 #include "linbox/algorithms/cra-domain.h"
-#include "linbox/algorithms/cra-full-multip.h"
-#include "linbox/algorithms/cra-early-multip.h"
+#include "linbox/algorithms/cra-builder-full-multip.h"
+#include "linbox/algorithms/cra-builder-early-multip.h"
 #include "linbox/randiter/random-prime.h"
 #include "linbox/solutions/solve.h"
 #include "linbox/solutions/methods.h"
-#include <vector>
+#include "linbox/solutions/hadamard-bound.h"
 
 namespace LinBox
 {
@@ -60,12 +60,12 @@ namespace LinBox
 		size_t* P = new size_t[N];
 		size_t* Qt = new size_t[N-1];
 
-		FFPACK::LUdivine ((typename Field::Father_t)F, FFLAS::FflasUnit, FFLAS::FflasNoTrans, N-1, N,
+		FFPACK::LUdivine (F, FFLAS::FflasUnit, FFLAS::FflasNoTrans, N-1, N,
 				  A, lda, P, Qt);
 		typename Field::Element d;
 
 		// Multiplying all (N-1) first pivots)
-		F.init(d, 1UL);
+		F.assign(d, F.one);
 		for (size_t i=0; i<N-1; ++i)
 			F.mulin (d, *(A + i*(lda+1)));
 
@@ -76,14 +76,14 @@ namespace LinBox
 			F.negin(d);
 
 		// Trick: instead of Right-Trans, do Left-NoTrans in order to use inc*
-		FFPACK::applyP ((typename Field::Father_t)F, FFLAS::FflasLeft, FFLAS::FflasNoTrans,
+		FFPACK::applyP (F, FFLAS::FflasLeft, FFLAS::FflasNoTrans,
 				1, 0,(int) N-1, b, incb, P);
-		FFPACK::applyP ((typename Field::Father_t)F, FFLAS::FflasLeft, FFLAS::FflasNoTrans,
+		FFPACK::applyP (F, FFLAS::FflasLeft, FFLAS::FflasNoTrans,
 				1, 0,(int) N-1, c, incc, P);
-		FFLAS::ftrsv ((typename Field::Father_t)F, FFLAS::FflasUpper, FFLAS::FflasTrans,
+		FFLAS::ftrsv (F, FFLAS::FflasUpper, FFLAS::FflasTrans,
 			      FFLAS::FflasUnit,
 			      N, A, lda, b, (int)incb);
-		FFLAS::ftrsv ((typename Field::Father_t)F, FFLAS::FflasUpper, FFLAS::FflasTrans,
+		FFLAS::ftrsv (F, FFLAS::FflasUpper, FFLAS::FflasTrans,
 			      FFLAS::FflasUnit,
 			      N, A, lda, c, (int)incc);
 
@@ -109,8 +109,8 @@ namespace LinBox
 		{}
 
 		template<class Field>
-		std::vector<typename Field::Element>&
-		operator () (std::vector<typename Field::Element>& dd,
+		IterationResult
+		operator () (BlasVector<Field>& dd,
 			     const Field& F) const
 		{
 
@@ -133,87 +133,10 @@ namespace LinBox
 			F.divin (dd[1], s2p);
 			//tim.stop();
 			//std::cerr<<"doubleDetModp took "<<tim.usertime()<<std::endl;
-			return dd;
+			return IterationResult::CONTINUE;
 		}
 	};
 
-	/* Computes the actual Hadamard bound of the matrix A by taking the minimum
-	 * of the column-wise and the row-wise euclidean norm.
-	 *
-	 * A is supposed to be over integers.
-	 *
-	 */
-	// should use multiprec floating pt arith, wait, maybe not!
-	template<class BlackBox>
-	integer& HadamardBound (integer& hadamarBound, const BlackBox& A)
-	{
-
-
-		integer res1 = 1;
-		integer res2 = 1;
-		integer temp;
-
-		typename BlackBox::ConstRowIterator rowIt;
-		typename BlackBox::ConstRow::const_iterator col;
-
-		for( rowIt = A.rowBegin(); rowIt != A.rowEnd(); ++rowIt ) {
-			temp = 0;
-			for( col = rowIt->begin(); col != rowIt->end(); ++col )
-				temp += static_cast<integer>((*col)) * (*col);
-			res1 *= temp;
-		}
-		res1 = sqrt(res1);
-
-		typename BlackBox::ConstColIterator colIt;
-		typename BlackBox::ConstCol::const_iterator row;
-
-		for( colIt = A.colBegin(); colIt != A.colEnd(); ++colIt ) {
-			temp = 0;
-			for( row = colIt->begin(); row != colIt->end(); ++row )
-				temp += static_cast<integer>((*row)) * (*row);
-			res2 *= temp;
-		}
-		res2 = sqrt(res2);
-
-		return hadamarBound = MIN(res1, res2);
-	}
-
-#if 0
-	template<class BlackBox>
-	double& HadamardBound (double& hadamarBound, const BlackBox& A)
-	{
-
-
-		double res1 = 0;
-		double res2 = 0;
-		integer temp;
-
-		typename BlackBox::ConstRowIterator rowIt;
-		typename BlackBox::ConstRow::const_iterator col;
-
-		for( rowIt = A.rowBegin(); rowIt != A.rowEnd(); ++rowIt ) {
-			temp = 0;
-			for( col = rowIt->begin(); col != rowIt->end(); ++col )
-				temp += static_cast<integer>((*col)) * (*col);
-			res1 += log ((double)temp);
-		}
-		res1 = res1/2;
-
-		typename BlackBox::ConstColIterator colIt;
-		typename BlackBox::ConstCol::const_iterator row;
-
-		for( colIt = A.colBegin(); colIt != A.colEnd(); ++colIt ) {
-			temp = 0;
-			for( row = colIt->begin(); row != colIt->end(); ++row )
-				temp += static_cast<integer>((*row)) * (*row);
-			res2 += log((double)temp);
-		}
-		res2 = res2/2;
-
-		return hadamarBound = MIN(res1, res2);
-	}
-#endif
-
 	/* Given a (N+1) x N full rank matrix
 	 * [ A ]
 	 * [ b ]
@@ -241,21 +164,21 @@ namespace LinBox
 		typename BlackBox::Field F = A.field();
 		IntegerDoubleDetIteration<BlackBox> iteration(A, s1, s2);
 		// 0.7213475205 is an upper approximation of 1/(2log(2))
-		RandomPrimeIterator genprime( 25-(int)ceil(log((double)A.rowdim())*0.7213475205));
+                typedef Givaro::ModularBalanced<double> Field;
+                PrimeIterator<IteratorCategories::HeuristicTag> genprime(FieldTraits<Field>::bestBitSize(A.coldim()));
 
-		std::vector<typename BlackBox::Field::Element> dd;
+		BlasVector<typename BlackBox::Field> dd(A.field());
 		if (proof) {
-			integer bound;
 			double logbound;
 			//Timer t_hd,t_cra;
 			//t_hd.clear();
 			//t_hd.start();
-			HadamardBound (bound, A);
-			logbound = (logtwo (bound) - logtwo (MIN(abs(s1),abs(s2))))*0.693147180559945;
+			size_t bound = HadamardBound(A);
+			logbound = (bound - logtwo (MIN(abs(s1),abs(s2))));
 			//t_hd.stop();
 			//std::cerr<<"Hadamard bound = : "<<logbound<<" in "<<t_hd.usertime()<<"s"<<std::endl;
 
-			ChineseRemainder <FullMultipCRA <Modular <double> > > cra(logbound);
+			ChineseRemainder <CRABuilderFullMultip <Givaro::Modular <double> > > cra(logbound);
 
 			//t_hd.clear();
 			//t_cra.start();
@@ -265,7 +188,7 @@ namespace LinBox
 
 		}
 		else {
-			ChineseRemainder <EarlyMultipCRA <Modular<double> > > cra(4UL);
+			ChineseRemainder <CRABuilderEarlyMultip <Field> >  cra(LINBOX_DEFAULT_EARLY_TERMINATION_THRESHOLD);
 			cra (dd, iteration, genprime);
 		}
 		F.mul (d1, dd[0], s1);
@@ -295,7 +218,7 @@ namespace LinBox
 		const size_t N = A.coldim();
 		BlasMatrix<typename BlackBox::Field> B (A.field(),N,N);
 		typename BlackBox::Field::Element den1, den2;
-		std::vector<typename BlackBox::Field::Element> x1(N);
+		BlasVector<typename BlackBox::Field> x1(A.field(),N);
 		for (size_t i=0; i<N; ++i){
 			x1[i] = 0;
 			for (size_t j=0; j<N; ++j)
@@ -304,7 +227,7 @@ namespace LinBox
 		// 		for (size_t i=0; i<N; ++i)
 		// 			B.setEntry (i, N-1, b[i]);
 
-		std::vector<typename BlackBox::Field::Element> c(N);
+		BlasVector<typename BlackBox::Field> c(A.field(),N);
 		for (size_t i=0; i<N; ++i)
 			c[i] = A.getEntry (N, i);
 		//Timer tim;
@@ -314,7 +237,7 @@ namespace LinBox
 		//tim.stop();
 		//std::cerr<<"Solve took "<<tim.usertime()<<std::endl;
 
-		den1 = den1;
+		// den1 = den1;
 		// Should work:
 		// den (y[n]) = den (-den1/x[n]) = x[n]
 		den2 = -x1[N-1];
@@ -330,12 +253,10 @@ namespace LinBox
 
 #endif // __LINBOX_doubledet_H
 
-
-// vim:sts=8:sw=8:ts=8:noet:sr:cino=>s,f0,{0,g0,(0,:0,t0,+0,=s
 // Local Variables:
 // mode: C++
-// tab-width: 8
+// tab-width: 4
 // indent-tabs-mode: nil
-// c-basic-offset: 8
+// c-basic-offset: 4
 // End:
-
+// vim:sts=4:sw=4:ts=4:et:sr:cino=>s,f0,{0,g0,(0,\:0,t0,+0,=s
