@@ -125,7 +125,7 @@ namespace LinBox
 		Element& powtwo(Element& h, const size_t log_h) const
 		{
 			h = 1;
-			if (log_h <= 0) return h;
+			// if (log_h <= 0) return h;
 			h<<=log_h;
 			return h;
 		}
@@ -146,7 +146,6 @@ namespace LinBox
 
 			if (!fastEEA (ai,bi,ci,di,m,log_m,x,bound, log_bound)) return false;
 
-			int K=0;
 
 			Element init_ainext = cur_ainext;
 			Element init_qinext = cur_qinext;
@@ -154,6 +153,7 @@ namespace LinBox
 			Element init_ri = cur_ri;
 			/* correction if $K <> bound = 2^log_bound */
 			if (cur_rinext > 0) {
+				int K=0;
 				while (cur_ainext <= d_bound) {
 					++K;
 					Element tmp;
@@ -350,15 +350,15 @@ namespace LinBox
 				aistar=distar=1;
 				bistar=cistar=0;
 
-				Element mstar = m >> (long unsigned int) lambda;
-				Element nstar = n >> (long unsigned int) lambda;
+				Element mstar = m >> (uint64_t) lambda;
+				Element nstar = n >> (uint64_t) lambda;
 
 				size_t log_mstar = 2*h+1;
 				if (nstar > 0) if (!fastEEA(aistar, bistar, cistar, distar, mstar, log_mstar,nstar, powh, h)) return false;
 
-				int K=2;//reatreat steps;
 
 				if ((aistar > 1) && (distar > 0)) { //we have to go back 2 steps
+					int K=2;//reatreat steps;
 					Element aprev,bprev,cprev,dprev;
 					for (int i=0; i < K; ++i) {
 						prevEEA(aprev,bprev,cprev,dprev,aistar,bistar,cistar,distar);
@@ -403,9 +403,9 @@ namespace LinBox
 				Element ri = cur_ri;
 				Element rinext = cur_rinext;
 
-				size_t log_m;
 
 				if ((rinext > 0) && (cur_ainext <= powh)){
+				size_t log_m;
 					log_m = rinext.bitsize()-1;
 					Element m2, n2;
 					m2 = rinext;
@@ -424,12 +424,12 @@ namespace LinBox
 
 					int k = (int)a1.bitsize()-1 ;
 					int _k;
-					if (h-k > 2)
-						_k = (int)(h-k-2);
+					if ((int)h-k > 2)
+						_k = ((int)h-k-2);
 					else _k = 0;
 					if (n2 >0) {
 						if (a1 < powh) {
-							if (!fastEEA(a2,b2,c2,d2,m2,log_m,n2, powtwo(sqrth,_k), _k)) return false;
+							if (!fastEEA(a2,b2,c2,d2,m2,log_m,n2, powtwo(sqrth,(size_t)_k), (size_t)_k)) return false;
 						}
 						else {
 							ai = a1; bi = b1; ci=c1; di = d1;
@@ -545,7 +545,7 @@ namespace LinBox
 				}
 
 				Element ainext, binext, cinext,dinext;
-				ainext=dinext=1;
+				dinext=1;
 				binext=cinext=0;
 				ainext = cur_ainext;
 				int K=-1;
@@ -659,7 +659,7 @@ namespace LinBox
 			a = tmpa; b = tmpb; c = tmpc; d = tmpd;
 		}
 
-		QMatrix& max(const QMatrix& max1, const QMatrix max2) {
+		QMatrix /* & */ max(const QMatrix& max1, const QMatrix max2) {
 			if (max1.q >= max2.q) return QMatrix(max1);
 			else return QMatrix(max2);
 		}
@@ -695,7 +695,7 @@ namespace LinBox
 			if (_size+1 < _maxSize) {
 				this->push_back(bottom);
 				return false;
-				++_size;
+				// ++_size;
 			}
 			else {
 				if (!this->empty()) {
@@ -733,6 +733,7 @@ namespace LinBox
 
 		FastMaxQRationalReconstruction(const Ring& Z) :
 			FastRationalReconstruction<Ring>(Z), _intRing(Z)
+			,c(0)
 		{}
 
 		bool reconstructRational(Element& a, Element& b, const Element& x, const Element& m) const
@@ -743,8 +744,8 @@ namespace LinBox
 
 		bool reconstructRational(Element& a, Element& b, const Element& x, const Element& m, const Element& a_bound) const
 		{
-			bool res= false;
-			return res = FastRationalReconstruction<Ring>::reconstructRational(a,b,x,m,a_bound);
+			// bool res= false;
+			return /*  res = */ FastRationalReconstruction<Ring>::reconstructRational(a,b,x,m,a_bound);
 		}
 
 	protected:
@@ -795,13 +796,13 @@ namespace LinBox
 			d = maxQ.a;
 
 			//Element T = m.bitsize();int c = 5;
-			if (maxQ.q.bitsize() > T.bitsize() + c) return true;
+			if (maxQ.q.bitsize() > T.bitsize() + (size_t)c) return true;
 			else return false;
 		}
 
 		bool classicQMaxEEA(Element& ai, Element& bi, Element& ci, Element& di, const Element& r0, const Element& r1,const Element& powh, myQueue<Ring >&  queueMax, QMatrix<Ring>& maxQ) const
 		{
-			if (maxQ.q.bitsize() > T.bitsize() + c) return true;
+			if (maxQ.q.bitsize() > T.bitsize() + (size_t) c) return true;
 			Element ri, rinext;
 			ri = r0; rinext = r1;
 			ai =di = 1;
@@ -820,7 +821,7 @@ namespace LinBox
 				if (queueMax.pushpop(top, newQ)) {
 					if (maxQ.q < top.q) {
 						maxQ = top;
-						if (maxQ.q.bitsize() > T.bitsize() + c) return true;
+						if (maxQ.q.bitsize() > T.bitsize() + (size_t)c) return true;
 					}
 				}
 
@@ -865,7 +866,7 @@ namespace LinBox
 				 const Element& m, const size_t d, const Element& n,
 				 const Element& powh, const size_t& h, myQueue<Ring >&  queueMax, QMatrix<Ring>& maxQ) const
 		{
-			if (maxQ.q.bitsize() > T.bitsize() + c) return true;
+			if (maxQ.q.bitsize() > T.bitsize() + (size_t)c) return true;
 			ai=Element(1);
 			di=Element(1);
 			bi=Element(0);
@@ -882,7 +883,7 @@ namespace LinBox
 				if (queueMax.pushpop(top, newQ)) {
 					if (maxQ.q < top.q) {
 						maxQ = top;
-						if (maxQ.q.bitsize() > T.bitsize() + c) return true;
+						if (maxQ.q.bitsize() > T.bitsize() + (size_t)c) return true;
 					}
 				}
 				return true;
@@ -911,7 +912,7 @@ namespace LinBox
 					if (queueMax.pushpop(top, newQ)) {
 						if (maxQ.q < top.q) {
 							maxQ = top;
-							if (maxQ.q.bitsize() > T.bitsize() + c) return true;
+							if (maxQ.q.bitsize() > T.bitsize() + (size_t)c) return true;
 						}
 					}
 
@@ -934,7 +935,7 @@ namespace LinBox
 					if (queueMax.pushpop(top, newQ)) {
 						if (maxQ.q < top.q) {
 							maxQ = top;
-							if (maxQ.q.bitsize() > T.bitsize() + c) return true;
+							if (maxQ.q.bitsize() > T.bitsize() + (size_t)c) return true;
 						}
 					}
 				}
@@ -960,14 +961,14 @@ namespace LinBox
 				aistar=distar=1;
 				bistar=cistar=0;
 
-				Element mstar = m >> (long unsigned int) lambda;
-				Element nstar = n >> (long unsigned int) lambda;
+				Element mstar = m >> (uint64_t) lambda;
+				Element nstar = n >> (uint64_t) lambda;
 
 				size_t log_mstar = 2*h+1;
 
 				queueMax._maxSize +=2;
 				if (nstar > 0) if (!fastQMaxEEA(aistar, bistar, cistar, distar, mstar, log_mstar,nstar, powh, h, queueMax, maxQ)) return false;
-				if (maxQ.q.bitsize() > T.bitsize() + c) return true;
+				if (maxQ.q.bitsize() > T.bitsize() + (size_t)c) return true;
 				if (queueMax._size > 1) {
 					queueMax.pop_back();
 					QMatrix<Ring> Q_i_2 (queueMax.back());
@@ -1016,14 +1017,13 @@ namespace LinBox
 				this->powtwo(sqrth, logsqrth);
 
 				if (!fastQMaxEEA(a1,b1,c1,d1,m,d,n,sqrth, logsqrth, queueMax, maxQ)) return false;
-				if (maxQ.q.bitsize() > T.bitsize() + c) return true;
+				if (maxQ.q.bitsize() > T.bitsize() + (size_t)c) return true;
 
 				ai = a1; bi = b1; ci=c1; di = d1;
 
 				Element ri = cur_ri;
 				Element rinext = cur_rinext;
 
-				size_t log_m;
 
 				myQueue<Ring> queueTmp (queueMax._maxSize);
 				QMatrix<Ring> maxQTmp(_intRing);
@@ -1034,10 +1034,11 @@ namespace LinBox
 					if (queueMax.pushpop(top, newQ)) {
 						if (maxQ.q < top.q) {
 							maxQ = top;
-							if (maxQ.q.bitsize() > T.bitsize() + c) return true;
+							if (maxQ.q.bitsize() > T.bitsize() + (size_t)c) return true;
 						}
 					}
 
+				size_t log_m;
 					log_m = rinext.bitsize()-1;
 					Element m2, n2;
 					m2 = rinext;
@@ -1062,7 +1063,7 @@ namespace LinBox
 					if (n2 >0) {
 						if (a1 < powh) {
 							if (!fastQMaxEEA(a2,b2,c2,d2,m2,log_m,n2,
-									 this->powtwo(sqrth,_k), _k, queueTmp,maxQTmp))
+									 this->powtwo(sqrth,(size_t)_k), (size_t)_k, queueTmp,maxQTmp))
 								return false;
 						}
 						else {
@@ -1137,7 +1138,7 @@ namespace LinBox
 				if (maxQ.q < maxQTmp.q) {
 					maxQTmp.leftmultiply(Q_i);
 					maxQ = maxQTmp;
-					if (maxQ.q.bitsize() > T.bitsize() + c) return true;
+					if (maxQ.q.bitsize() > T.bitsize() + (size_t)c) return true;
 				}
 				int K=0;
 				QMatrix<Ring > Q(_intRing);
@@ -1171,7 +1172,7 @@ namespace LinBox
 						if (queueMax.pushpop(top, Q)) {
 							if (maxQ.q < top.q) {
 								maxQ = top;
-								if (maxQ.q.bitsize() > T.bitsize() + c) return true;
+								if (maxQ.q.bitsize() > T.bitsize() + (size_t)c) return true;
 							}
 						}
 					}
@@ -1213,7 +1214,7 @@ namespace LinBox
 			}
 
 			Element ainext, binext, cinext,dinext;
-			ainext=dinext=1;
+			dinext=1;
 			binext=cinext=0;
 			ainext = cur_ainext;
 			int K=-1;
@@ -1234,7 +1235,7 @@ namespace LinBox
 					if (queueMax.pushpop(top, Q)) {
 						if (maxQ.q < top.q) {
 							maxQ = top;
-							if (maxQ.q.bitsize() > T.bitsize() + c) return true;
+							if (maxQ.q.bitsize() > T.bitsize() + (size_t)c) return true;
 						}
 					}
 
@@ -1277,11 +1278,10 @@ namespace LinBox
 
 #endif //__LINBOX_fast_reconstruction_H
 
-// vim:sts=8:sw=8:ts=8:noet:sr:cino=>s,f0,{0,g0,(0,:0,t0,+0,=s
 // Local Variables:
 // mode: C++
-// tab-width: 8
+// tab-width: 4
 // indent-tabs-mode: nil
-// c-basic-offset: 8
+// c-basic-offset: 4
 // End:
-
+// vim:sts=4:sw=4:ts=4:et:sr:cino=>s,f0,{0,g0,(0,\:0,t0,+0,=s
