@@ -1,4 +1,3 @@
-
 /* linbox/algorithms/mg-block-lanczos.h
  * Copyright (C) 2002 Bradford Hovinen
  *
@@ -39,6 +38,7 @@
 #include "linbox/vector/vector-domain.h"
 #include "linbox/blackbox/archetype.h"
 #include "linbox/solutions/methods.h"
+#include "linbox/matrix/dense-matrix.h"
 
 // I'm putting everything inside the LinBox namespace so that I can drop all of
 // this in to LinBox easily at a later date, without any messy porting.
@@ -46,6 +46,7 @@
 namespace LinBox
 {
 
+
 	/** \brief Block Lanczos iteration
 	 *
 	 * This is a blocked version of the iteration given in @ref LanczosSolver. The
@@ -62,7 +63,7 @@ namespace LinBox
 	 * Currently, only dense vectors are supported for this iteration, and it is
 	 * unlikely any other vector archetypes will be supported in the future.
 	 */
-	template <class Field, class Matrix = BlasMatrix<typename Field::Element> >
+	template <class Field, class Matrix = BlasMatrix<Field,typename RawVector<typename Field::Element >::Dense> >
 	class MGBlockLanczosSolver {
 	public:
 
@@ -73,11 +74,13 @@ namespace LinBox
 		 * @param traits @ref SolverTraits  structure describing user
 		 *               options for the solver
 		 */
-		MGBlockLanczosSolver (const Field &F, const BlockLanczosTraits &traits) :
-			_traits (traits), _field (F), _VD (F), _MD (F), _randiter (F), _block (traits.blockingFactor ())
+		MGBlockLanczosSolver (const Field &F, const Method::BlockLanczos &traits) :
+			_traits (traits), _field (&F), _VD (F), _MD (F), _randiter (F)
+			,_AV(F), _VTAV(F), _AVTAVSST_VTAV(F), _matT(F), _DEF(F)
+            , _x(F), _y(F), _b(F), _tmp(F), _tmp1(F), _matM(F)
+			, _block (traits.blockingFactor)
 		{
 			init_temps ();
-			_field.init (_one, 1);
 		}
 
 		/** Constructor with a random iterator
@@ -86,11 +89,13 @@ namespace LinBox
 		 *               options for the solver
 		 * @param r Random iterator to use for randomization
 		 */
-		MGBlockLanczosSolver (const Field &F, const BlockLanczosTraits &traits, typename Field::RandIter r) :
-			_traits (traits), _field (F), _VD (F), _MD (F), _randiter (r), _block (traits.blockingFactor ())
+		MGBlockLanczosSolver (const Field &F, const Method::BlockLanczos &traits, typename Field::RandIter r) :
+			_traits (traits), _field (&F), _VD (F), _MD (F), _randiter (r)
+			,_AV(F), _VTAV(F), _AVTAVSST_VTAV(F), _matT(F), _DEF(F)
+            , _x(F), _y(F), _b(F), _tmp(F), _tmp1(F), _matM(F)
+			, _block (traits.blockingFactor)
 		{
 			init_temps ();
-			_field.init (_one, 1);
 		}
 
 		/** Solve the linear system Ax = b.
@@ -218,20 +223,25 @@ namespace LinBox
 		// Initialize the temporaries used in computation
 		void init_temps ();
 
+		// are we public?
+		inline const Field & field() const { return *_field; }
+
 		// Private variables
 
-		const BlockLanczosTraits _traits;
-		const Field              &_field;
+		const Method::BlockLanczos _traits;
+		const Field              *_field;
 		VectorDomain<Field>       _VD;
 		MatrixDomain<Field>       _MD;
 		typename Field::RandIter  _randiter;
 
 		// Temporaries used in the computation
 
-		Matrix  _matV[3];             // n x N
+		// Matrix  _matV[3];             // n x N
+		std::vector<Matrix> _matV;
 		Matrix  _AV;               // n x N
 		Matrix  _VTAV;             // N x N
-		Matrix  _Winv[2];          // N x N
+		// Matrix  _Winv[2];          // N x N
+		std::vector<Matrix> _Winv ;
 		Matrix  _AVTAVSST_VTAV;    // N x N
 		Matrix  _matT;                // N x N
 		Matrix  _DEF;              // N x N
@@ -244,7 +254,6 @@ namespace LinBox
 		mutable Matrix _tmp;       // N x <=N
 		mutable Matrix _tmp1;      // N x <=N
 
-		typename Field::Element   _one;
 
 		std::vector<size_t>       _indices;          // N
 
@@ -286,11 +295,10 @@ namespace LinBox
 
 #endif // __LINBOX_mg_block_lanczos_H
 
-// vim:sts=8:sw=8:ts=8:noet:sr:cino=>s,f0,{0,g0,(0,:0,t0,+0,=s
 // Local Variables:
 // mode: C++
-// tab-width: 8
+// tab-width: 4
 // indent-tabs-mode: nil
-// c-basic-offset: 8
+// c-basic-offset: 4
 // End:
-
+// vim:sts=4:sw=4:ts=4:et:sr:cino=>s,f0,{0,g0,(0,\:0,t0,+0,=s
