@@ -1,5 +1,3 @@
-/* -*- mode: C++; tab-width: 4; indent-tabs-mode: t; c-basic-offset: 4 -*- */
-// vim:sts=4:sw=4:ts=4:noet:sr:cino=>s,f0,{0,g0,(0,\:0,t0,+0,=s
 /*
  * Copyright (C) 2016 Romain Lebreton, Pascal Giorgi
  *
@@ -154,9 +152,6 @@ namespace LinBox {
 			//uint64_t wi = 1;
 			Element wi = 1;
 
-			// Precomp Quo(2^32,p)
-			Compute_t invp; fld->precomp_p(invp);
-
 			if (ln>0){
 //				using simd=Simd<uint32_t>;
 //				using vect_t =typename simd::vect_t;
@@ -297,3 +292,11 @@ namespace LinBox {
 }
 
 #endif // __LINBOX_polynomial_fft_init_H
+
+// Local Variables:
+// mode: C++
+// tab-width: 4
+// indent-tabs-mode: nil
+// c-basic-offset: 4
+// End:
+// vim:sts=4:sw=4:ts=4:et:sr:cino=>s,f0,{0,g0,(0,\:0,t0,+0,=s
