@@ -63,24 +63,24 @@ namespace Givaro
         a = rr.nume();
         b = rr.deno();
     }
-    
+
     inline void reconstructRational (Integer& a, Integer& b, const Integer& x, const Integer& m)
     {
         reconstructRational(a,b, x, m, Givaro::sqrt(m));
     }
-    
+
     inline bool reconstructRational (Integer& a, Integer& b,
                                      const Integer& x, const Integer& m,
                                      const Integer& a_bound, const Integer& b_bound)
     {
         Integer bound = x/b_bound;
 			// if (bound>a_bound) std::cerr << "a_bound: " << a_bound << ", x/b_bound: " << bound << std::endl;
-        
+
         reconstructRational(a,b,x,m, (bound>a_bound?bound:a_bound));
-        
+
         return b<= b_bound;
     }
-    
+
 }
 
 
@@ -92,7 +92,7 @@ namespace LinBox
 	}
 
 
-    
+
 
 	/*! \brief Limited doc so far.
 	 * Used, for instance, after LiftingContainer.
@@ -379,7 +379,7 @@ namespace LinBox
 				else {
 					int status= (int)Givaro::reconstructRational(tmp_num, tmp_den, *res_p, modulus, numbound, denbound);
 					if (!status) {
-						commentator().report() 
+						commentator().report()
 						<< "ERROR in reconstruction ? (1)\n" << std::endl;
 #ifdef DEBUG_RR
 						std::cout<<" try to reconstruct :\n";
@@ -766,7 +766,6 @@ namespace LinBox
 		template<class Vector1>
 		bool getRational3(Vector1& num, Integer& den) const
 		{
-
 #ifdef RSTIMING
 			ttRecon.clear();
 			tRecon.start();
@@ -809,8 +808,7 @@ namespace LinBox
 			ttRecon += tRecon;
 #endif
 #ifdef LIFTING_PROGRESS
-			Commentator lifting_commentator;
-			lifting_commentator().start("Padic Lifting","LinBox::LiftingContainer",_lcontainer.length());
+			commentator().start("Padic Lifting","LinBox::LiftingContainer",_lcontainer.length());
 #endif
 #if 0
 			Timer eval_horner,eval_horn;
@@ -821,7 +819,7 @@ namespace LinBox
 			for (size_t i=0 ; iter != _lcontainer.end() && iter.next(digit_approximation[(size_t)i]);++i) {
 
 #ifdef LIFTING_PROGRESS
-				lifting_commentator().progress(i);
+				commentator().progress(i);
 #endif
 #if 0
 				eval_horn.start();
@@ -834,7 +832,7 @@ namespace LinBox
 			}
 
 #ifdef LIFTING_PROGRESS
-			lifting_commentator().stop ("Done", "Done", "LinBox::LinBox::LiftingContainer");
+			commentator().stop ("Done", "Done", "LinBox::LinBox::LiftingContainer");
 #endif
 
 			// problem occured during lifting
@@ -917,28 +915,28 @@ namespace LinBox
 			/*
 			 * dumb rational reconstruction (this is just for timing comparison)
 			 */
-#if 0
-			{
-				Timer dumb_ratrecon;
-				dumb_ratrecon.start();
-				Vector den_r(num.size()), num_r(num.size());
-				typename Vector::iterator   iter_a  = real_approximation.begin();
-				typename Vector::iterator   iter_n  = num_r.begin();
-				typename Vector::iterator   iter_d  = den_r.begin();
+// #if 0
+// 			{
+// 				Timer dumb_ratrecon;
+// 				dumb_ratrecon.start();
+// 				Vector den_r(num.size()), num_r(num.size());
+// 				typename Vector::iterator   iter_a  = real_approximation.begin();
+// 				typename Vector::iterator   iter_n  = num_r.begin();
+// 				typename Vector::iterator   iter_d  = den_r.begin();
 
-				for (size_t i=0; iter_a != real_approximation.end(); ++iter_a, ++ iter_n, ++iter_d, ++i){
-					if (!Givaro::reconstructRational(*iter_n, *iter_d,
-								    *iter_a, modulus, numbound, denbound))
-					{
-						commentator().report()
-						<< "ERROR in reconstruction ?\n" << std::endl;
-					}
-				}
-				dumb_ratrecon.stop();
-				std::cout<<"full rational reconstruction : "<<dumb_ratrecon.usertime()<<std::endl;
-			}
+// 				for (size_t i=0; iter_a != real_approximation.end(); ++iter_a, ++ iter_n, ++iter_d, ++i){
+// 					if (!Givaro::reconstructRational(*iter_n, *iter_d,
+// 								    *iter_a, modulus, numbound, denbound))
+// 					{
+// 						commentator().report()
+// 						<< "ERROR in reconstruction ?\n" << std::endl;
+// 					}
+// 				}
+// 				dumb_ratrecon.stop();
+// 				std::cout<<"full rational reconstruction : "<<dumb_ratrecon.usertime()<<std::endl;
+// 			}
 
-#endif
+// #endif
 			/*
 			 * Rational Reconstruction of each coefficient according to a common denominator
 			 */
@@ -952,7 +950,7 @@ namespace LinBox
 
 			Vector denominator(_r,num.size());
 
-			int counter=0;
+                        int counter=0;
 			typename Vector::iterator   iter_approx = real_approximation.begin();
 			typename Vector1::iterator  iter_num    = num.begin();
 			typename Vector::iterator   iter_denom  = denominator.begin();
@@ -993,26 +991,26 @@ namespace LinBox
 					_r.mulin(common_den, *iter_denom);
 					idx_last_den=(int)i;
 					counter++;
-#if 0
-					if (i != size-1)
-					{
-						//if (! _r.isUnit(*iter_denom))
-						counter++;
+// #if 0
+// 					if (i != size-1)
+// 					{
+// 						//if (! _r.isUnit(*iter_denom))
+// 						counter++;
 
-						_r.quoin(denbound , *iter_denom);
-						_r.mul(bound, denbound,numbound);
-						_r.mulin(bound,two);
-						_r.div(tmp,modulus,prime);
-						while(tmp > bound) {
-							_r.assign(modulus,tmp);
-							_r.div(tmp,modulus,prime);
-						}
-						_r.mod(tmp , *iter_denom , modulus);
-						_r.modin(common_den_mod_prod , modulus);
-						_r.mulin(common_den_mod_prod , tmp);
-						_r.modin(common_den_mod_prod , modulus);
-					}
-#endif
+// 						_r.quoin(denbound , *iter_denom);
+// 						_r.mul(bound, denbound,numbound);
+// 						_r.mulin(bound,two);
+// 						_r.div(tmp,modulus,prime);
+// 						while(tmp > bound) {
+// 							_r.assign(modulus,tmp);
+// 							_r.div(tmp,modulus,prime);
+// 						}
+// 						_r.mod(tmp , *iter_denom , modulus);
+// 						_r.modin(common_den_mod_prod , modulus);
+// 						_r.mulin(common_den_mod_prod , tmp);
+// 						_r.modin(common_den_mod_prod , modulus);
+// 					}
+// #endif
 
 				}
 
@@ -1024,7 +1022,6 @@ namespace LinBox
 				_r.mulin(tmp,denominator[(size_t)i]);
 			}
 
-
 #if 0
 			typename Vector1::reverse_iterator rev_iter_num   = num.rbegin();
 			typename Vector::reverse_iterator  rev_iter_denom = denominator.rbegin();
@@ -1068,7 +1065,7 @@ namespace LinBox
 			if (den > 0) lcm(init_den,den,den_app);
 			_r. assign(den, _r.one);
 #ifdef DEBUG_RR
-			cout << "debug: den " << den;
+			std::cout << "debug: den " << den;
 #endif
 			Integer prime = _lcontainer.prime();                    // prime used for lifting
 			Vector digit(_lcontainer.size());                       // to store next digit
@@ -1764,13 +1761,13 @@ namespace LinBox
 				for (size_t i=1;i< k+1;++i){
 					Lattice.setEntry(i,i, mod );
 					_r.convert(tmp, real_approximation[bad_num_index+i-1]);
-					Lattice.setEntry(0,i,fplll::Z_NR<mpz_t>(tmp.get_mpz()));
+					Lattice.setEntry(0,i,tmp);
 				}
 
 
 				// ratio to check the validity of the denominator compare to the entries in the reduced lattice
 				integer ratio;
-				ratio=100L;
+				ratio=100;
 
 				// reduce the lattice using LLL algorithm
 				Timer chrono;
@@ -2091,7 +2088,7 @@ namespace LinBox
 
 				// ratio to check the validity of the denominator compare to the entries in the reduced lattice
 				integer ratio;
-				ratio=100L;
+				ratio=100;
 
 				// reduce the lattice using LLL algorithm
 				Timer chrono;
@@ -2248,12 +2245,10 @@ namespace LinBox
 #undef DEF_THRESH
 #endif //__LINBOX_reconstruction_H
 
-
-// vim:sts=8:sw=8:ts=8:noet:sr:cino=>s,f0,{0,g0,(0,:0,t0,+0,=s
 // Local Variables:
 // mode: C++
-// tab-width: 8
+// tab-width: 4
 // indent-tabs-mode: nil
-// c-basic-offset: 8
+// c-basic-offset: 4
 // End:
-
+// vim:sts=4:sw=4:ts=4:et:sr:cino=>s,f0,{0,g0,(0,\:0,t0,+0,=s
