@@ -30,15 +30,17 @@
 
 //#define __CHECK_SIGMA_BASIS
 
-#include "linbox/algorithms/sigma-basis.h"
+
+#include "linbox/matrix/polynomial-matrix.h"
+#include "linbox/algorithms/polynomial-matrix/order-basis.h"
 #include "linbox/blackbox/block-hankel.h"
-//#include "linbox/blackbox/block-toeplitz.h"
 #include "linbox/blackbox/compose.h"
 #include "linbox/vector/vector-domain.h"
 
 //#define PADEMATRIX
 
 
+
 namespace LinBox
 {
 
@@ -48,9 +50,11 @@ namespace LinBox
 	public:
 		typedef _Field                    Field;
 		typedef typename Field::Element Element;
-		typedef BlasMatrix<Element> Coefficient;
+		typedef BlasMatrix<Field> Coefficient;
+                typedef PolynomialMatrix<PMType::matfirst,PMStorage::plain, Field> PMatrix;
+                
 	private:
-		Field                   _field;
+		const Field           *_field;
 		VectorDomain<Field>    _VD;
 		BlockHankel<Field>    *_H1;
 		BlockHankel<Field>    *_T1;
@@ -62,11 +66,11 @@ namespace LinBox
 		size_t      _block;
 	public:
 
-		// Constructor from a stl vector of BlasMatrix reprenting
+		// Constructor from a stl vector of BlasMatrix representing
 		// all different elements in the Hankel representation
 		// vector is of odd size and represent the 1st column and last row append together
-		BlockHankelInverse(const Field &F, const std::vector<BlasMatrix<Element> > &P) :
-			_field(F), _VD(F), _BMD(F)
+		BlockHankelInverse(const Field &F, const std::vector<BlasMatrix<Field> > &P) :
+			_field(&F), _VD(F), _BMD(F)
 		{
 			//write_maple("UAV",P);
 
@@ -75,8 +79,7 @@ namespace LinBox
 			size_t rowblock = (deg+1)>>1;
 			size_t colblock = rowblock;
 			_row = _col = colblock*block;
-			Element one;
-			_field.init(one, 1UL);
+			Element one=field().one;
 			_numblock=rowblock;
 			_block= block;
 
@@ -84,31 +87,31 @@ namespace LinBox
 			// compute right and left matrix Pade Approximant of P of order deg+2 and deg
 
 
-#ifndef PADEMATRIX
-			// construct the matrix polynomial serie  [ P(x)^T  I ]^T and [ P^T(x) I ]
-			std::vector<Coefficient> RightPowerSerie(deg+2), LeftPowerSerie(deg+2), LPS(deg+2), RPS(deg+2);
+			// construct the matrix power series
+                        // LeftPowerSerie = [ P(x)^T  I ]^T
+                        // RighPowerSerie = [ P(x)   I ]^T --> this should be transposed but we use left order basis instead of right one
+			
+                        PMatrix LeftPowerSerie (field(),2*block,block,deg+2);
+                        PMatrix RightPowerSerie(field(),block,2*block,deg+2);                        
 			for (size_t i=0;i< deg+2; ++i){
-				LeftPowerSerie[i]  = Coefficient(2*block, block); LPS[i]= Coefficient(2*block, block);
-				RightPowerSerie[i] = Coefficient(block, 2*block); RPS[i]= Coefficient(block, 2*block);
 				if (i <deg) {
 					for (size_t j=0;j<block;++j)
 						for (size_t k=0;k<block;++k){
-							LeftPowerSerie[i].setEntry(j,k,  P[i].getEntry(j,k));LPS[i].setEntry(j,k,  P[i].getEntry(j,k));
-							RightPowerSerie[i].setEntry(j,k, P[i].getEntry(j,k));RPS[i].setEntry(j,k,  P[i].getEntry(j,k));
-						}
+				                        field().assign(LeftPowerSerie.ref(j,k,i),  P[i].getEntry(j,k));				
+							field().assign(RightPowerSerie.ref(k,j,i), P[i].getEntry(j,k));
+                                                }
 				}
 			}
 			for (size_t j=0;j<block;++j){
-				LeftPowerSerie[0].setEntry(block+j, j , one);	LPS[0].setEntry(block+j, j , one);
-				RightPowerSerie[0].setEntry(j, block+j, one);   RPS[0].setEntry(j, j+block , one);
+                                field().assign(LeftPowerSerie.ref (block+j, j ,0), one);
+                                field().assign(RightPowerSerie.ref(block+j, j, 0), one);
 			}
-
-			//write_maple("Sx",LeftPowerSerie);
-
-			SigmaBasis<Field> SBL(F, LeftPowerSerie);
-			SigmaBasis<Field> SBR(F, RightPowerSerie);
-
-			std::vector<Coefficient > LP1, RP1, LP2, RP2;
+                        
+			OrderBasis<Field> OB(F);
+                        
+                        PMatrix LP1(field(),2*block,2*block,deg), RP1(field(),2*block,2*block,deg);
+                        PMatrix LP2(field(),2*block,2*block,deg+2), RP2(field(),2*block,2*block,deg+2);
+                        
 			size_t two_n= block<<1;
 			std::vector<size_t> dlp1(two_n,0), drp1(two_n,0), dlp2(two_n,0), drp2(two_n,0);
 
@@ -118,56 +121,24 @@ namespace LinBox
 				dlp2[i]=1;
 				drp2[i]=1;
 			}
-
-
+                        
 			// Compute the sigma basis
 			//Timer chrono;
 			//chrono.start();
-			SBL.multi_left_basis  (LP1, deg-1, dlp1, LP2, deg+1, dlp2);
-			SBR.multi_right_basis (RP1, deg-1, drp1, RP2, deg+1, drp2);
-			//chrono.stop();
-			//std::cout<<"SigmaBasis Computation : "<<chrono<<"\n";
-			//SBL.printTimer();
-
-			//SBL.left_basis  (LP1, deg-1, dlp1);
-			//SBR.right_basis (RP1, deg-1, drp1);
-			//SBL.left_basis  (LP2, deg+1, dlp2);
-			//SBR.right_basis (RP2, deg+1, drp2);
-
-			std::vector<BlasMatrix<Element> > SLP1, SLP2, SRP1, SRP2;
+                        OB.PM_Basis(LP1, LeftPowerSerie, deg-1, dlp1);
+                        OB.PM_Basis(LP2, LeftPowerSerie, deg+1, dlp2); // MUST BE OPTIMIZED -> modify power serie thanks to LP1 and compute small basis
+                        OB.PM_Basis(RP1, RightPowerSerie, deg-1, dlp1);
+                        OB.PM_Basis(RP2, RightPowerSerie, deg+1, dlp2); // MUST BE OPTIMIZED -> modify power serie thanks to RP1 and compute small basis
 
+                        
+			std::vector<BlasMatrix<Field> > SLP1, SLP2, SRP1, SRP2;
 			extractLeftSigma  (SLP1, LP1, dlp1, block);
 			extractLeftSigma  (SLP2, LP2, dlp2, block);
+			extractTransposedRightSigma (SRP1, RP1, drp1, block);
+			extractTransposedRightSigma (SRP2, RP2, drp2, block);
+                        
 
-			extractRightSigma (SRP1, RP1, drp1, block);
-			extractRightSigma (SRP2, RP2, drp2, block);
-
-#else
-			size_t two_n= block<<1;
-			std::vector<size_t> dlp(two_n,0), drp(two_n,0);
-			for (size_t i=block;i<two_n;++i){
-				dlp[i]=1;
-				drp[i]=1;
-			}
-
-			Coefficient ZeroSerie(block, block);
-			std::vector<Coefficient> Serie(deg+2, ZeroSerie);
-			for (size_t i=0;i<deg;++i)
-				for (size_t j=0;j<block;++j)
-					for (size_t k=0;k<block;++k)
-						Serie[i].setEntry(j,k, P[i].getEntry(j,k));
-
-			SigmaBasis<Field> SBL(F, Serie);
-			SigmaBasis<Field> SBR(F, Serie);
-
-			std::vector<BlasMatrix<Element> > SLP1, SLP2, SRP1, SRP2;
-			SBL.multi_left_PadeMatrix  (SLP1, deg-1, SLP2, deg+1, dlp);
-			SBR.multi_right_PadeMatrix (SRP1, deg-1, SRP2, deg+1, drp);
-
-#endif
-
-
-			BlasMatrix<Element> Res(block,block), Inv(block, block);
+			BlasMatrix<Field> Res(field(),block,block), Inv(field(),block, block);
 
 			int singular;
 			// Normalization of SLP2 (V*)
@@ -229,17 +200,18 @@ namespace LinBox
 			   write_maple("V",SRP2);
 			   */
 
-			std::vector<Coefficient> rev_poly(SRP2.size());
+			Coefficient Zero2(field());
+			std::vector<Coefficient> rev_poly(SRP2.size(),Zero2);
 			for (size_t i=0;i<SRP2.size();++i)
 				rev_poly[i]= SRP2[SRP2.size()-1-i];
 
 			rev_poly.erase(rev_poly.begin());
 
 
-			_H1 = new BlockHankel<Field>  (_field, rev_poly, BlockHankelTag::up); // V
+			_H1 = new BlockHankel<Field>  (field(), rev_poly, BlockHankelTag::up); // V
 
 			rev_poly.resize(SRP2.size());
-			const BlasMatrix<Element> Zero(block,block);
+			const BlasMatrix<Field> Zero(field(),block,block);
 			for (size_t i=0;i<SRP1.size();++i)
 				rev_poly[i]= SRP1[SRP1.size()-1-i];
 			rev_poly[SRP1.size()]= Zero;
@@ -247,15 +219,16 @@ namespace LinBox
 
 
 
-			_H2 = new BlockHankel<Field>  (_field, rev_poly, BlockHankelTag::up); // Q
+			_H2 = new BlockHankel<Field>  (field(), rev_poly, BlockHankelTag::up); // Q
 
-			_T1 = new BlockHankel<Field>  (_field, SLP1, BlockHankelTag::up); // Qstar
+			_T1 = new BlockHankel<Field>  (field(), SLP1, BlockHankelTag::up); // Qstar
 
 			SLP2.erase(SLP2.begin());
-			_T2 = new BlockHankel<Field>  (_field, SLP2, BlockHankelTag::up); // Vstar
+			_T2 = new BlockHankel<Field>  (field(), SLP2, BlockHankelTag::up); // Vstar
 
 		}
 
+		//!@bug copy constructor ??
 
 		~BlockHankelInverse()
 		{
@@ -276,7 +249,7 @@ namespace LinBox
 			// reverse y according to block structure
 			for (size_t i=0; i< _numblock; ++i)
 				for (size_t j=0;j<_block;++j){
-					_field.assign(rev_y[(_numblock-i-1)*_block+j], y[i*_block+j]);
+					field().assign(rev_y[(_numblock-i-1)*_block+j], y[i*_block+j]);
 				}
 
 			_T1->apply(z1, rev_y);
@@ -299,24 +272,16 @@ namespace LinBox
 
 		size_t coldim() const { return _col;}
 
-		const Field& field() const { return _field;}
+		const Field& field() const { return *_field;}
 
 	protected:
 
-		void extractLeftSigma(std::vector<BlasMatrix<Element> >              &S,
-				      std::vector<BlasMatrix<Element> >      &SigmaBase,
+		void extractLeftSigma(std::vector<BlasMatrix<Field> >                &S,
+				      PMatrix                                &SigmaBase,
 				      std::vector<size_t>                       &defect,
 				      size_t                                      block) const
 		{
-#if 0
-			std::cout<<"Pade Approximant:\n defect= ";
-			for (size_t i=0;i<defect.size();++i)
-				std::cout<<defect[i]<<",";
-			std::cout<<"\n";
-			for (size_t i=0;i<SigmaBase.size();++i)
-				SigmaBase[i].write(std::cout,_field);
-#endif
-
+                        
 			// take the block rows which have lowest defect
 			// compute permutation such that first block rows have lowest defect
 			std::vector<size_t> Perm(2*block);
@@ -336,19 +301,6 @@ namespace LinBox
 			for (size_t i=0;i<SigmaBase.size();++i)
 				_BMD.mulin_right(BPerm,SigmaBase[i]);
 
-#if 0
-			// take the  row of SigmaBase s.t. SigmaBase[0] is invertible
-			std::vector<size_t> notnull(block);
-			size_t idx=0;
-			for (size_t i=0; i<2*block;++i){
-				for (size_t j=0;j<2*block;++j)
-					if (!_field.isZero(SigmaBase[0].getEntry(i,j))){
-						notnull[idx]=i;
-						++idx;
-						break;
-					}
-			}
-#endif
 			size_t max=defect[0];
 			for (size_t i=0;i<block;++i)
 				if (defect[i] > max)
@@ -356,33 +308,23 @@ namespace LinBox
 					max=defect[i];
 
 			// prepare S to receive the sigma base
-			const BlasMatrix<Element> Zero(block,block);
+			const BlasMatrix<Field> Zero(field(),block,block);
 			S.resize(max+1, Zero);
 
 			// extract the sigma base
 			for (size_t k=0;k<S.size();++k){
 				for(size_t i=0;i<block;++i)
-					for (size_t j=0;j<block;++j)
-						//S[k].setEntry(i,j, SigmaBase[k].getEntry(notnull[i],j));
+					for (size_t j=0;j<block;++j)			
 						S[k].setEntry(i,j, SigmaBase[k].getEntry(i,j));
 			}
 		}
 
 
-		void extractRightSigma(std::vector<BlasMatrix<Element> >            &S,
-				       std::vector<BlasMatrix<Element> >    &SigmaBase,
-				       std::vector<size_t>                     &defect,
-				       size_t                                    block) const
+		void extractTransposedRightSigma(std::vector<BlasMatrix<Field> >            &S,
+                                                 std::vector<BlasMatrix<Field> >    &SigmaBase,
+                                                 std::vector<size_t>                     &defect,
+                                                 size_t                                    block) const
 		{
-#if 0
-			std::cout<<"Pade Approximant:\n defect= ";
-			for (size_t i=0;i<defect.size();++i)
-				std::cout<<defect[i]<<",";
-			std::cout<<"\n";
-
-			for (size_t i=0;i<SigmaBase.size();++i)
-				SigmaBase[i].write(std::cout,_field);
-#endif
 
 			// take the m rows which have lowest defect
 			// compute permutation such that first block rows have lowest defect
@@ -403,27 +345,13 @@ namespace LinBox
 			for (size_t i=0;i<SigmaBase.size();++i)
 				_BMD.mulin_right(BPerm,SigmaBase[i]);
 
-			/*
-			// take the  row of SigmaBase s.t. SigmaBase[0] is invertible
-			std::vector<size_t> notnull(block);
-			size_t idx=0;
-			for (size_t i=0; i<2*block;++i){
-			for (size_t j=0;j<2*block;++j)
-			if (!_field.isZero(SigmaBase[0].getEntry(i,j))){
-			notnull[idx]=i;
-			++idx;
-			break;
-			}
-			}
-			*/
-
 			size_t max=defect[0];
 			for (size_t i=0;i<block;++i)
 				if (defect[i] > max)
 					max=defect[i];
 
 			// prepare S to receive the sigma base
-			const BlasMatrix<Element> Zero(block,block);
+			const BlasMatrix<Field> Zero(field(),block,block);
 			S.resize(max+1, Zero);
 
 			// extract the sigma base
@@ -456,26 +384,26 @@ namespace LinBox
 				for (size_t i=0;i<m-1;++i){
 					std::cout<<"[";
 					for (size_t j=0;j<n-1;++j)
-						_field.write(std::cout,P[k].getEntry(i,j))<<",";
-					_field.write(std::cout, P[k].getEntry(i,n-1))<<"] , ";
+						field().write(std::cout,P[k].getEntry(i,j))<<",";
+					field().write(std::cout, P[k].getEntry(i,n-1))<<"] , ";
 				}
 				std::cout<<"[";
 				for (size_t j=0;j<n-1;++j)
-					_field.write(std::cout,P[k].getEntry(m-1,j))<<",";
-				_field.write(std::cout, P[k].getEntry(m-1,n-1))<<"]]) , ";
+					field().write(std::cout,P[k].getEntry(m-1,j))<<",";
+				field().write(std::cout, P[k].getEntry(m-1,n-1))<<"]]) , ";
 			}
 
 			std::cout<<"Matrix([";
 			for (size_t i=0;i<m-1;++i){
 				std::cout<<"[";
 				for (size_t j=0;j<n-1;++j)
-					_field.write(std::cout,P[P.size()-1].getEntry(i,j))<<",";
-				_field.write(std::cout, P[P.size()-1].getEntry(i,n-1))<<"] , ";
+					field().write(std::cout,P[P.size()-1].getEntry(i,j))<<",";
+				field().write(std::cout, P[P.size()-1].getEntry(i,n-1))<<"] , ";
 			}
 			std::cout<<"[";
 			for (size_t j=0;j<n-1;++j)
-				_field.write(std::cout,P[P.size()-1].getEntry(m-1,j))<<",";
-			_field.write(std::cout, P[P.size()-1].getEntry(m-1,n-1))<<"]])]; \n";
+				field().write(std::cout,P[P.size()-1].getEntry(m-1,j))<<",";
+			field().write(std::cout, P[P.size()-1].getEntry(m-1,n-1))<<"]])]; \n";
 		}
 
 
@@ -485,16 +413,13 @@ namespace LinBox
 
 }// end of namespace LinBox
 
-#undef PADEMATRIX
 
 #endif //__LINBOX_block_hankel_inverse_H
 
-
-// vim:sts=8:sw=8:ts=8:noet:sr:cino=>s,f0,{0,g0,(0,:0,t0,+0,=s
 // Local Variables:
 // mode: C++
-// tab-width: 8
+// tab-width: 4
 // indent-tabs-mode: nil
-// c-basic-offset: 8
+// c-basic-offset: 4
 // End:
-
+// vim:sts=4:sw=4:ts=4:et:sr:cino=>s,f0,{0,g0,(0,\:0,t0,+0,=s
