@@ -28,6 +28,7 @@
 
 #include "linbox/blackbox/blackbox-interface.h"
 #include "toeplitz.h" // we inherit everything from ntl-toeplitz
+// #include "linbox/vector/blas-vector.h"
 
 //#define DBGMSGS 1
 
@@ -45,17 +46,35 @@ namespace LinBox
 		typedef _Field Field;
 		typedef typename Field::Element Element;
 
+		typedef Toeplitz<_Field> TBase;
+		typedef TBase Father_t;
+                using TBase::P;
+                using TBase::rowDim;
+                using TBase::colDim;
+                using TBase::sysDim;
+                using TBase::shape;
+                using TBase::pdata;
+                using TBase::rpdata;
+                using TBase::field_;
+                using TBase::field;
+
+
 		template<typename _Tp1>
 		struct rebind
 		{ typedef Hankel<_Tp1> other; };
 
 		//------- CONSTRUCTORS AND DESTRUCTORS
 
-		~Hankel();                // Destructor
-		Hankel();                 // Zero Param Constructor
-		Hankel( const Field F,    // Cnstr. with Field and STL vec. of elems
-			const std::vector<Element>&v);
+		~Hankel();
+		// Hankel();// : Toeplitz<_Field>(){}
+		Hankel( const Field F) : Father_t (F) {};
+		// Cnstr. with Field and STL vec. of elems
+		Hankel( const Field F,    const std::vector<Element>&v);// : Toeplitz<_Field>(F, v){}
+		Hankel( const BlasVector<Field>&v);// : Toeplitz<_Field>(F, v){}
+
+		Hankel( const Field F,    size_t n);// : Toeplitz<_Field>(F, n) {}
 		//	  Hankel(char *dataFileName ); // read from a file
+		//void init( const Field F,    size_t n) { Toeplitz<_Field>::init(F, n) }
 
 		//------- INHERITED READ-ONLY ACCESSOR, and OBSERVER METHODS
 
@@ -66,11 +85,6 @@ namespace LinBox
 		 *      inline size_t sysdim() const;// Max of rows & columns;
 		 */
 
-		//------- MUTATOR METHODS
-
-		void setToUniModUT() ;      // Convert to UTriang matrix with det 1
-		void setToUniModLT() ;      // Convert to LTriang matrix with det 1
-
 		//------ SERVICE METHODS
 		template<class OutVector, class InVector>
 		OutVector& apply( OutVector &v_out, const InVector& v_in) const;
@@ -86,12 +100,10 @@ namespace LinBox
 
 #endif //__LINBOX_ntl_hankel_H
 
-
-// vim:sts=8:sw=8:ts=8:noet:sr:cino=>s,f0,{0,g0,(0,:0,t0,+0,=s
 // Local Variables:
 // mode: C++
-// tab-width: 8
+// tab-width: 4
 // indent-tabs-mode: nil
-// c-basic-offset: 8
+// c-basic-offset: 4
 // End:
-
+// vim:sts=4:sw=4:ts=4:et:sr:cino=>s,f0,{0,g0,(0,\:0,t0,+0,=s
