@@ -1,22 +1,22 @@
 /* linbox/blackbox/zo-gf2.h
  * Copyright (C) 2009,2010 The LinBox group
  *
- * Time-stamp: <23 Jun 10 16:02:02 Jean-Guillaume.Dumas@imag.fr>
+ * Time-stamp: <04 Dec 17 11:25:43 Jean-Guillaume.Dumas@imag.fr>
+ *
  *
- * 
  * ========LICENCE========
  * This file is part of the library LinBox.
- * 
+ *
  * LinBox is free software: you can redistribute it and/or modify
  * it under the terms of the  GNU Lesser General Public
  * License as published by the Free Software Foundation; either
  * version 2.1 of the License, or (at your option) any later version.
- * 
+ *
  * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  * Lesser General Public License for more details.
- * 
+ *
  * You should have received a copy of the GNU Lesser General Public
  * License along with this library; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
@@ -102,7 +102,7 @@ namespace LinBox
 		size_t coldim() const { return _coldim; }
 
 
-		void setEntry(size_t i, size_t j, const Element& v) ;
+		const Element& setEntry(size_t i, size_t j, const Element& v) ;
 		const Element& getEntry(size_t i, size_t j) const ;
 		Element& getEntry(Element&, size_t i, size_t j) const ;
 
@@ -119,31 +119,16 @@ namespace LinBox
 		 *  @return Reference to input stream
 		 */
 		std::istream &read (std::istream &is) ;
-		std::ostream& write (std::ostream& out, LINBOX_enum(Tag::FileFormat) format=Tag::FileFormat::Guillaume) const ;
+		std::ostream& write (std::ostream& out, Tag::FileFormat format=Tag::FileFormat::Guillaume) const ;
 
 		const Field& field() const { return *_field; }
 
 		template<typename _Tp1>
-		struct rebind {
-			typedef ZeroOne<_Tp1> other;
-			void operator() (other & Ap,
-					 const Self_t& A,
-					 const _Tp1& F)
-			{
-				// ZeroOne does not store any Field element
-			}
-		};
+		struct rebind;
 
 
 		template<typename _Tp1>
-		ZeroOne(ZeroOne<_Tp1>& A, const GF2 F2) :
-			Father_t(A.rowdim()), _rowdim(A.rowdim()), _coldim(A.coldim()), _nnz(0)
-		{
-			for(typename ZeroOne<_Tp1>::IndexIterator it = A.indexBegin();
-			    it != A.indexEnd(); ++it,++_nnz) {
-				this->operator[]( it->first ).push_back( it->second );
-			}
-		}
+		ZeroOne(const ZeroOne<_Tp1>& A, const GF2 F2);
 
 		size_t nnz() const { return _nnz; }
 		bool isRowSorted() const { return true; }
@@ -158,15 +143,20 @@ namespace LinBox
 		const Iterator Begin() const;
 		const Iterator End() const;
 
-		/** IndexIterator - Iterates through the i and j of the current element
+		/** IndexedIterator - Iterates through the i and j of the current element
 		 * and when accessed returns an STL pair containing the coordinates
 		 */
-		class IndexIterator;
-		IndexIterator indexBegin();
-		const IndexIterator indexBegin() const;
-		IndexIterator indexEnd();
-		const IndexIterator indexEnd() const;
+		class IndexedIterator;
+		IndexedIterator indexBegin();
+		const IndexedIterator indexBegin() const;
+		IndexedIterator indexEnd();
+		const IndexedIterator indexEnd() const;
 
+    protected:
+        // Merge A with self
+        // Warning: respective supports must be disjoint
+        template<typename _Tp1>
+        void augment(const ZeroOne<_Tp1>&);
 
 	private:
 		size_t _rowdim, _coldim, _nnz;
@@ -178,12 +168,10 @@ namespace LinBox
 
 #endif //__LINBOX_zo_gf2_H
 
-
-// vim:sts=8:sw=8:ts=8:noet:sr:cino=>s,f0,{0,g0,(0,:0,t0,+0,=s
 // Local Variables:
 // mode: C++
-// tab-width: 8
+// tab-width: 4
 // indent-tabs-mode: nil
-// c-basic-offset: 8
+// c-basic-offset: 4
 // End:
-
+// vim:sts=4:sw=4:ts=4:et:sr:cino=>s,f0,{0,g0,(0,\:0,t0,+0,=s
