@@ -1,4 +1,3 @@
-/* -*- mode: C++; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*
  * Copyright (C) 2013  Pascal Giorgi
  *                     Romain Lebreton
@@ -149,11 +148,14 @@ namespace LinBox{
 				//_rep.shrink_to_fit();
 			}
 			integer p;_fld->characteristic(p); size_t bb=p.bitsize(); if(bb>64) bb+=128; bb/=8;
+
+#ifdef MEM_PMBASIS
 			size_t mem=realmeminfo();
-			_store=s;
-			setsize(s);
 			ADD_MEM(realmeminfo());
 			DEL_MEM(mem);
+#endif			
+			_store=s;
+			setsize(s);
 		}
 
 		void changesize(size_t s){
@@ -279,6 +281,15 @@ namespace LinBox{
 		size_t storage() const {return _store;}
 		const Field& field()  const {return *_fld;}
 
+		void dump(std::ostream& os) const {
+			os<<_row<<" "<<_col<<" "<<_size<<std::endl;
+			for(size_t i=0;i<_row*_col;i++)
+				for(size_t k=0;k<_size;k++)
+					os<<get(i,k)<<" ";
+
+			os<<std::endl;
+		}
+		
 		std::ostream& write(std::ostream& os) const { return write(os,0,_size-1);}
 
                 std::ostream& write(std::ostream& os, size_t deg_min, size_t deg_max) const {
@@ -314,8 +325,8 @@ namespace LinBox{
                 	return os;
                 }
 
-		Element* getWritePointer(){return &_rep[0];}
-		const Element* getPointer() const {return &_rep[0];}
+		Element* getWritePointer(){return _rep.data();}
+		const Element* getPointer() const {return _rep.data();}
 
 		size_t realmeminfo()const {
 			return _row*_col*(_store*element_storage(field())+sizeof(Polynomial));}
@@ -511,6 +522,15 @@ namespace LinBox{
 		
 		inline const Field& field()  const {return *_fld;}
 
+		void dump(std::ostream& os) const {
+			os<<_row<<" "<<_col<<" "<<_size<<std::endl;
+			for(size_t k=0;k<_size;k++)
+				for(size_t i=0;i<_row*_col;i++)		
+					os<<get(i,k)<<" ";
+			os<<std::endl;
+		}
+
+		
 		std::ostream& write(std::ostream& os) const { return write(os,0,real_degree());}
 
                 std::ostream& write(std::ostream& os, size_t deg_min, size_t deg_max) const {
@@ -726,3 +746,11 @@ namespace LinBox{
 } //end of namespace LinBox
 
 #endif // __LINBOX_polynomial_matrix_H
+
+// Local Variables:
+// mode: C++
+// tab-width: 4
+// indent-tabs-mode: nil
+// c-basic-offset: 4
+// End:
+// vim:sts=4:sw=4:ts=4:et:sr:cino=>s,f0,{0,g0,(0,\:0,t0,+0,=s
