@@ -603,7 +603,7 @@ namespace LinBox
 		 * @warning if this is used to build a matrix and this matrix is "well formed",
 		 * it can be sped up (no checking that the entry already exists).
 		 */
-		void setEntry(const size_t &i, const size_t &j, const Element& e
+		const Element& setEntry(const size_t &i, const size_t &j, const Element& e
 			     )
 		{
 			linbox_check(i<_rownb);
@@ -612,7 +612,8 @@ namespace LinBox
 			linbox_check(consistent());
 
 			if (field().isZero(e)) {
-				return clearEntry(i,j);
+				clearEntry(i,j);
+                return e;
 			}
 
 			size_t * beg = &_colid[i*_maxc];
@@ -644,6 +645,7 @@ namespace LinBox
 			if (!found) {
 				insert(i,_maxc,j,e);
 			}
+			return e;
 		}
 
 #if 0
@@ -690,7 +692,7 @@ namespace LinBox
 		 * @param format Format with which to write
 		 */
 		std::ostream & write(std::ostream &os
-				     , LINBOX_enum(Tag::FileFormat) format = Tag::FileFormat::MatrixMarket) const
+				     , Tag::FileFormat format = Tag::FileFormat::MatrixMarket) const
 		{
 			return SparseMatrixWriteHelper<Self_t>::write(*this,os,format);
 		}
@@ -702,7 +704,7 @@ namespace LinBox
 		 * @return ref to \p is.
 		 */
 		std::istream& read (std::istream &is
-				    , LINBOX_enum(Tag::FileFormat) format = Tag::FileFormat::Detect)
+				    , Tag::FileFormat format = Tag::FileFormat::Detect)
 		{
 			return SparseMatrixReadHelper<Self_t>::read(*this,is,format);
 		}
@@ -763,8 +765,8 @@ namespace LinBox
 
 		// y= Ax
 		// y[i] = sum(A(i,j) x(j)
-		template<class Vector>
-		Vector& apply(Vector &y, const Vector& x, const Element & a ) const
+		template<class outVector, class inVector>
+		outVector& apply(outVector &y, const inVector& x, const Element & a ) const
 		{
 			// linbox_check(consistent());
 			prepare(field(),y,a);
@@ -1482,8 +1484,8 @@ namespace LinBox
 
 // Local Variables:
 // mode: C++
-// tab-width: 8
+// tab-width: 4
 // indent-tabs-mode: nil
-// c-basic-offset: 8
+// c-basic-offset: 4
 // End:
-// vim:sts=8:sw=8:ts=8:noet:sr:cino=>s,f0,{0,g0,(0,\:0,t0,+0,=s
+// vim:sts=4:sw=4:ts=4:et:sr:cino=>s,f0,{0,g0,(0,\:0,t0,+0,=s
