@@ -154,9 +154,9 @@ namespace LinBox
 		 * @param j Column number 0...coldim () - 1
 		 * @param a_ij Element to set
 		 */
-		void setEntry (size_t i, size_t j, const Element &a_ij)
+		const Element& setEntry (size_t i, size_t j, const Element &a_ij)
 		{
-			_Mat.setEntry (j, i, a_ij);
+			return _Mat.setEntry (j, i, a_ij);
 		}
 
 		/** Get a writeable reference to the entry in the (i, j) position.
@@ -339,9 +339,9 @@ namespace LinBox
 			return _Mat.write (stream);
 		}
 
-		inline void setEntry (size_t i, size_t j, const Element &a_ij)
+		inline const Element& setEntry (size_t i, size_t j, const Element &a_ij)
 		{
-			_Mat.setEntry (j, i, a_ij);
+			return _Mat.setEntry (j, i, a_ij);
 		}
 		inline Element &refEntry (size_t i, size_t j)
 		{
@@ -470,9 +470,9 @@ namespace LinBox
 			return _Mat.write (stream);
 		}
 
-		inline void setEntry (size_t i, size_t j, const Element &a_ij)
+		inline const Element& setEntry (size_t i, size_t j, const Element &a_ij)
 		{
-			_Mat.setEntry (j, i, a_ij);
+			return _Mat.setEntry (j, i, a_ij);
 		}
 		inline Element &refEntry (size_t i, size_t j)
 		{
@@ -581,9 +581,9 @@ namespace LinBox
 			return _Mat.write (stream);
 		}
 
-		inline void setEntry (size_t i, size_t j, const Element &a_ij)
+		inline const Element& setEntry (size_t i, size_t j, const Element &a_ij)
 		{
-			_Mat.setEntry (j, i, a_ij);
+			return _Mat.setEntry (j, i, a_ij);
 		}
 		inline Element &refEntry (size_t i, size_t j)
 		{
@@ -694,11 +694,10 @@ namespace LinBox
 
 #endif // __LINBOX_matrix_transpose_INL
 
-
 // Local Variables:
 // mode: C++
-// tab-width: 8
+// tab-width: 4
 // indent-tabs-mode: nil
-// c-basic-offset: 8
+// c-basic-offset: 4
 // End:
-// vim:sts=8:sw=8:ts=8:noet:sr:cino=>s,f0,{0,g0,(0,\:0,t0,+0,=s
+// vim:sts=4:sw=4:ts=4:et:sr:cino=>s,f0,{0,g0,(0,\:0,t0,+0,=s
