@@ -52,10 +52,10 @@ namespace LinBox
 		 */
 		LidiaGfqRandIter(const field& F,
 				 const integer& size = 0,
-				 const integer& seed = 0) :
+				 const uint64_t seed = 0) :
 			_size(size), _seed(seed) , GF(F)
 		{
-			if (_seed == integer(0)) _seed = time(NULL);
+			if (_seed == 0) _seed = time(NULL);
 
 			integer cardinality ;
 			F.cardinality(cardinality);
@@ -133,7 +133,7 @@ namespace LinBox
 		integer _size;
 
 		/// Seed
-		integer _seed;
+		uint64_t _seed;
 
 		field GF;
 
@@ -143,12 +143,10 @@ namespace LinBox
 
 #endif // __LINBOX_lidia_randiter_gfq_H
 
-
-// vim:sts=8:sw=8:ts=8:noet:sr:cino=>s,f0,{0,g0,(0,:0,t0,+0,=s
 // Local Variables:
 // mode: C++
-// tab-width: 8
+// tab-width: 4
 // indent-tabs-mode: nil
-// c-basic-offset: 8
+// c-basic-offset: 4
 // End:
-
+// vim:sts=4:sw=4:ts=4:et:sr:cino=>s,f0,{0,g0,(0,\:0,t0,+0,=s
