@@ -71,7 +71,7 @@ namespace LinBox
 	typename Blackbox::Field::Element &valence (typename Blackbox::Field::Element         &v,
 						    const Blackbox                               &A)
 	{
-		return valence(v, A, Method::Hybrid());
+		return valence(v, A, Method::Auto());
 	}
 
 	template<class Blackbox, class MyMethod>
@@ -96,11 +96,10 @@ namespace LinBox
 }
 
 #include <givaro/modular.h>
-#include <typeinfo>
 
 #include "linbox/ring/modular.h"
 #include "linbox/algorithms/cra-domain.h"
-#include "linbox/algorithms/cra-early-single.h"
+#include "linbox/algorithms/cra-builder-single.h"
 #include "linbox/randiter/random-prime.h"
 #include "linbox/algorithms/matrix-hom.h"
 
@@ -118,22 +117,22 @@ namespace LinBox
 
 
 		template<typename Field>
-		typename Field::Element& operator()(typename Field::Element& v, const Field& F) const
+		IterationResult operator()(typename Field::Element& v, const Field& F) const
 		{
 			commentator().start ("Givaro::Modular Valence", "Mvalence");
-			std::ostream& report = commentator().report (Commentator::LEVEL_IMPORTANT, INTERNAL_DESCRIPTION);
-			F.write(report) << std::endl;
+// 			std::ostream& report = commentator().report (Commentator::LEVEL_IMPORTANT, INTERNAL_DESCRIPTION);
+// 			F.write(report) << std::endl;
 			typedef typename Blackbox::template rebind<Field>::other FBlackbox;
-			report << typeid(A).name() << ", A is: " << A.rowdim() << 'x' << A.coldim() << std::endl;
+// 			report << typeid(A).name() << ", A is: " << A.rowdim() << 'x' << A.coldim() << std::endl;
 
 			FBlackbox Ap(A, F);
 
-			report << typeid(Ap).name() << ", Ap is: " << Ap.rowdim() << 'x' << Ap.coldim() << std::endl;
+// 			report << typeid(Ap).name() << ", Ap is: " << Ap.rowdim() << 'x' << Ap.coldim() << std::endl;
 
 			valence( v, Ap, M);
-			F.write( F.write(report << "one valence: ", v) << " mod " ) << std::endl;;
+// 			F.write( F.write(report << "one valence: ", v) << " mod " ) << std::endl;;
 			commentator().stop ("done", NULL, "Mvalence");
-			return v;
+			return IterationResult::CONTINUE;
 		}
 	};
 
@@ -145,12 +144,13 @@ namespace LinBox
 	{
 		commentator().start ("Integer Valence", "Ivalence");
 #if __LINBOX_SIZEOF_LONG == 8
-		RandomPrimeIterator genprime( 31 );
-		ChineseRemainder< EarlySingleCRA< Givaro::Modular< int64_t> > > cra(3UL);
+		typedef Givaro::Modular<int64_t> Field;
 #else
-		RandomPrimeIterator genprime( 26 );
-		ChineseRemainder< EarlySingleCRA< Givaro::Modular<double> > > cra(3UL);
+		typedef Givaro::ModularBalanced<double> Field;
 #endif
+                PrimeIterator<IteratorCategories::HeuristicTag> genprime(FieldTraits<Field>::bestBitSize(A.rowdim()));
+		ChineseRemainder< CRABuilderEarlySingle<Field> > cra(LINBOX_DEFAULT_EARLY_TERMINATION_THRESHOLD);
+
 		IntegerModularValence<Blackbox,MyMethod> iteration(A, M);
 		cra(V, iteration, genprime);
 		commentator().stop ("done", NULL, "Ivalence");
@@ -158,6 +158,39 @@ namespace LinBox
 	}
 
 
+    template<class Blackbox>
+    typename Blackbox::Field::Element &squarizeValence(
+        typename Blackbox::Field::Element	&val_A,
+        const Blackbox						&A,
+        size_t method=0) {
+            // method:	0 is automatic
+            // 			1 is aat
+            // 			2 is ata
+
+        if (A.rowdim() == A.coldim()) {
+            return valence(val_A, A);
+        } else {
+            if (! method) {
+                if (A.rowdim() > A.coldim())
+                    method=2;
+                else
+                    method=1;
+            }
+            Transpose<Blackbox> T(&A);
+            if (method==2) {
+                Compose< Transpose<Blackbox>, Blackbox > C (&T, &A);
+                std::clog << "A^T A is " << C.rowdim() << " by " << C.coldim() << std::endl;
+                return valence(val_A, C);
+            } else {
+                Compose< Blackbox, Transpose<Blackbox> > C (&A, &T);
+                std::clog << "A A^T is " << C.rowdim() << " by " << C.coldim() << std::endl;
+                return valence(val_A, C);
+            }
+        }
+    }
+
+
+
 } //End of LinBox
 
 
@@ -269,7 +302,7 @@ namespace LinBox
 			typedef Givaro::Modular<int32_t> Field;
 			typedef typename MatrixHomTrait<Blackbox, Field>::value_type FBlackbox;
 			size_t d;
-			RandomPrimeIterator g; g.template setBitsField<Field>();
+			PrimeIterator<IteratorCategories::HeuristicTag> g(FieldTraits<Field>::bestBitSize(A.coldim()));
 			Field::Element v;
 			++g;
 			Field F((int32_t)*g);
@@ -293,7 +326,7 @@ namespace LinBox
 			typedef Givaro::Modular<int32_t> Field;
 			typedef typename MatrixHomTrait<Blackbox, Field>::value_type FBlackbox;
 
-			RandomPrimeIterator rg; rg.template setBitsField<Field>();
+			PrimeIterator<IteratorCategories::HeuristicTag> rg(FieldTraits<Field>::bestBitSize(A.coldim()));
 			Givaro::ZRing<Integer> Z;
 			BlasVector<Givaro::ZRing<Integer> > Lv(Z), Lm(Z);
 			size_t d1; Field::Element v; integer im = 1;
@@ -334,11 +367,10 @@ namespace LinBox
 } //End of LinBox
 #endif //__LINBOX_valence_H
 
-
 // Local Variables:
 // mode: C++
-// tab-width: 8
+// tab-width: 4
 // indent-tabs-mode: nil
-// c-basic-offset: 8
+// c-basic-offset: 4
 // End:
-// vim:sts=8:sw=8:ts=8:noet:sr:cino=>s,f0,{0,g0,(0,\:0,t0,+0,=s
+// vim:sts=4:sw=4:ts=4:et:sr:cino=>s,f0,{0,g0,(0,\:0,t0,+0,=s
