@@ -109,7 +109,7 @@ namespace LinBox
 			if( this->sin->eof() ) return END_OF_FILE;
 			if( !this->sin->good() ) return BAD_FORMAT;
 
-			n -= _base;
+			n -= (size_t)_base;
 			m = currentRow;
 			--colsLeft;
 
@@ -136,12 +136,10 @@ namespace LinBox
 
 #endif // __LINBOX_format_sparse_row_H
 
-
-// vim:sts=8:sw=8:ts=8:noet:sr:cino=>s,f0,{0,g0,(0,:0,t0,+0,=s
 // Local Variables:
 // mode: C++
-// tab-width: 8
+// tab-width: 4
 // indent-tabs-mode: nil
-// c-basic-offset: 8
+// c-basic-offset: 4
 // End:
-
+// vim:sts=4:sw=4:ts=4:et:sr:cino=>s,f0,{0,g0,(0,\:0,t0,+0,=s
