@@ -32,10 +32,10 @@
 #ifndef __LINBOX_subvector_H
 #define __LINBOX_subvector_H
 
-#include "linbox/vector/subiterator.h"
+//#include "linbox/vector/subiterator.h" BB:not used here
 #include <iterator>
-#include "linbox/vector/vector-traits.h"
 #include <stdexcept>
+#include "linbox/vector/vector-traits.h"
 
 namespace LinBox
 {
@@ -82,9 +82,18 @@ namespace LinBox
 
 		template<class Vector>
 		Subvector (Vector& v, size_type start, size_type stride, size_type length) :
-			_begin (iterator (v.begin() + start, stride)),
-			_end   (iterator (v.begin() + start + (stride * length), stride))
+			_begin (iterator (v.begin() + (ptrdiff_t)start, (ptrdiff_t)stride)),
+			_end   (iterator (v.begin() + (ptrdiff_t)start + (ptrdiff_t)(stride * length), (ptrdiff_t)stride))
+		{
+		}
+
+#if 0
+		template<lass otherIterator>
+		Subvector (otheriterator Begin, size_type start, size_type stride, size_type length) :
+			_begin (iterator (Begin + (ptrdiff_t)start, (ptrdiff_t)stride)),
+			_end   (iterator (Begin + (ptrdiff_t)start + (ptrdiff_t)(stride * length), (ptrdiff_t)stride))
 		{}
+#endif
 
 		Subvector(iterator Begin, iterator End) :
 			_begin(Begin), _end(End)
@@ -115,8 +124,8 @@ namespace LinBox
 
 		// Element access
 
-		reference       operator[] (size_type n)       { return _begin[n]; }
-		const_reference operator[] (size_type n) const { return _begin[n]; }
+		reference       operator[] (size_type n)       { return _begin[(ptrdiff_t)n]; }
+		const_reference operator[] (size_type n) const { return const_cast<iterator*>(&_begin)->operator[]((ptrdiff_t)n); }
 
 		// the method "at" does appear to be implemented
 		// in the gnu implementation of the STL
@@ -131,6 +140,8 @@ namespace LinBox
 
 		const_reference at(size_type n) const
 		{
+std::cerr << "n: " << n << std::endl;
+std::cerr << "_end-_begin: " << (int)(_end-_begin) << std::endl;
 			const_iterator p = _begin + n;
 			if ( _begin <= p && p < _end)
 				return *p;
@@ -184,10 +195,10 @@ namespace LinBox
 		Subvector& operator=(const Subvector& sub)
 		{ _begin = sub._begin; _end = sub._end; return *this; }
 
-		size_type size      (void) const { return _end - _begin; }
-		bool      empty     (void) const { return _end == _begin; }
-		size_type max_size  (void) const { return _end - _begin; }
-		//size_type capacity(void) const { return _end - _begin; }
+		size_type size      (void) const { return (size_t)(_end -  _begin); }
+		bool      empty     (void) const { return (size_t)(_end == _begin); }
+		size_type max_size  (void) const { return (size_t)(_end -  _begin); }
+		//size_type capacity(void) const { return (size_t)(_end -  _begin); }
 
 		// Swap
 		void swap (Subvector& x)
@@ -231,9 +242,11 @@ namespace LinBox
 } // namespace LinBox
 
 
+//! @warning why do we need this ?
 namespace std
 {
 
+#if 0
 	template<class _Tp>
 	void swap (_Tp&, _Tp&)
 #ifdef __GNUC__
@@ -245,6 +258,7 @@ namespace std
 #endif
 #endif
 	    ;
+#endif
 
 
 
@@ -262,12 +276,10 @@ namespace std
 
 #endif //__LINBOX_subvector_H
 
-
-// vim:sts=8:sw=8:ts=8:noet:sr:cino=>s,f0,{0,g0,(0,:0,t0,+0,=s
 // Local Variables:
 // mode: C++
-// tab-width: 8
+// tab-width: 4
 // indent-tabs-mode: nil
-// c-basic-offset: 8
+// c-basic-offset: 4
 // End:
-
+// vim:sts=4:sw=4:ts=4:et:sr:cino=>s,f0,{0,g0,(0,\:0,t0,+0,=s
