@@ -30,10 +30,15 @@ LOCAL_DIR="$JENKINS_DIR"/local
 export PATH=$PATH:/usr/local/bin:"$LOCAL_DIR/$CXX/bin"
 echo $PATH
 # Add specific locations (if needed)
-export LD_LIBRARY_PATH="$LD_LIBRARY_PATH":/usr/local/lib:"$LOCAL_DIR/$CXX/lib":"$LOCAL_DIR/$CXX/withSSE/lib"
+export LD_LIBRARY_PATH="$LD_LIBRARY_PATH":/usr/local/lib:"$LOCAL_DIR/$CXX/lib":"$LOCAL_DIR/$CXX/lib"
 echo "LD_LIBRARY_PATH = $LD_LIBRARY_PATH"
-export PKG_CONFIG_PATH="$LOCAL_DIR/$CXX/withSSE/lib/pkgconfig"
+export PKG_CONFIG_PATH="$LOCAL_DIR/$CXX/lib/pkgconfig"
 echo "PKG_CONFIG_PATH = $PKG_CONFIG_PATH"
+VERSION=`grep VERSION Makefile.am  | sed s/VERSION=// `
+echo "VERSION = $VERSION"
+distdir=linbox-$VERSION
+distarchive=$distdir.tar.gz
+echo "distarchive = $distarchive"
 
 # Where to install linbox binaries
 # Keep default for local installation.
@@ -60,20 +65,29 @@ if [ "$CXX" == "icpc" ]; then
 	source /usr/local/bin/compilervars.sh intel64
      fi
 fi
-# Particular case for Fedora: g++-6 <- g++
-if [[ "$ARCH" == "linbox-fedora-amd64" &&  "$CXX" == "g++-6" ]]; then
-    CXX="g++"
-    CC=gcc
-fi
 
 #==================================#
 # Automated installation and tests #
 #==================================#
 
-echo "|=== JENKINS AUTOMATED SCRIPT ===| ./autogen.sh CXX=$CXX CXXFLAGS=$CXXFLAGS --prefix=$PREFIX_INSTALL $LINBOX_NTLFLAG $LINBOX_FFLASFFPACKFLAG"
-./autogen.sh CXX=$CXX CXXFLAGS=$CXXFLAGS --prefix="$PREFIX_INSTALL" "$LINBOX_NTLFLAG" "$LINBOX_FFLASFFPACKFLAG"
+echo "|=== JENKINS AUTOMATED SCRIPT ===| ./autogen.sh CXX=$CXX CXXFLAGS=$CXXFLAGS"
+./autogen.sh CXX=$CXX CXXFLAGS=$CXXFLAGS --prefix="$PREFIX_INSTALL" "$LINBOX_NTLFLAG"
 V="$?"; if test "x$V" != "x0";then exit "$V"; fi
 
+echo "|=== JENKINS AUTOMATED SCRIPT ===| make dist"
+make dist
+V="$?"; if test "x$V" != "x0";then exit "$V"; fi
+
+rm -rf work
+mkdir -p work/builddir
+mv $distarchive work
+cd work
+tar zxvf $distarchive
+cd builddir
+
+echo "|=== JENKINS AUTOMATED SCRIPT ===| ../$distdir/configure CXX=$CXX CXXFLAGS=$CXXFLAGS --prefix=$PREFIX_INSTALL $LINBOX_NTLFLAG"
+../$distdir/configure CXX=$CXX CXXFLAGS=$CXXFLAGS --prefix="$PREFIX_INSTALL" "$LINBOX_NTLFLAG"
+
 echo "|=== JENKINS AUTOMATED SCRIPT ===| make install"
 make install
 V="$?"; if test "x$V" != "x0"; then exit "$V"; fi
@@ -85,3 +99,4 @@ echo "|=== JENKINS AUTOMATED SCRIPT ===| make examples"
 make examples
 V="$?"; if test "x$V" != "x0"; then exit "$V"; fi
 (cd examples && make clean)
+
